/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: LeanEdit.java,v 1.26 2001/11/08 22:27:35 max Exp $
 *
 */

import java.io.File;
import java.util.ResourceBundle;
import java.util.Enumeration;
import make.util.Debug;
import make.module.ModuleRepository;
import make.module.PathModuleRepository;
import make.module.ResourceLoader;
import make.module.ZipResourceLoader;
import org.leanedit.AbstractEditorApplication;
import org.leanedit.EditorSession;
import org.leanedit.LeanEditSession;
import org.leanedit.module.EditorModule;
import org.leanedit.module.ModuleManager;

public class LeanEdit extends AbstractEditorApplication {
    /*static**********************************/
    protected static final ResourceBundle resources
        = ResourceBundle.getBundle("org.leanedit.Editor");

    /*EditorApplication*************************/
    public void removeSession(EditorSession session) {
        super.removeSession(session);
        // exit the application if no sessions left
        if (!getSessions().hasMoreElements()) {
            // cleanup
            ModuleManager moduleManager = getModuleManager();
            for (Enumeration enum=moduleManager.getModuleNames();
                 enum.hasMoreElements();) {
                String name = (String)enum.nextElement();
                EditorModule module = moduleManager.getModule(name);
                uninstallModule(module);
            }
            // now stop the JVM
            System.exit(0);
        }
    }

    /*main*******************************************/
    public static void usage() {
        System.out.println("usage: LeanEdit [options] [filenames]");
        System.out.println("options:");
        System.out.println("\t-m");
        System.out.println("\t--moduledir DIR\tsets the path for the module repository");
        System.out.println("\t-l");
        System.out.println("\t--loadmodule FILE\tloads the module FILE");
        System.out.println("\t--openmodule MODULE[:PARAM]\topens a special window in the module MODULE");
    }

    public static void main(String[] args) {
        // check for help request before initializing GUI
        for (int z=0; z<args.length; z++) {
            if ("--help".equals(args[z]) || "-h".equals(args[z]) ||
                "-?".equals(args[z])) {
                usage(); System.exit(1);
            }
        }

        // redirect stderr for windows
        try {
            String operatingSystem = System.getProperty("os.name");
            if (operatingSystem.indexOf("indows")>0)
                Debug.logErr(new File("c:\\leanedit.log"));
        } catch (Exception e) {
            Debug.report(e);
        }

        try {
            LeanEdit leanEdit = new LeanEdit();

            // read global options from args
            int paramIndex = 0;
            while (paramIndex<args.length) {
                if (args[paramIndex].equals("-m") ||
                    args[paramIndex].equals("--moduledir")) {
                    // load new modules from directory
                    paramIndex++;
                    if (paramIndex>=args.length) {
                        System.err.println("missing directory after --moduledir");
                        System.exit(1);
                    }
                    File directory = new File(args[paramIndex++]);
                    if (!directory.isDirectory()) {
                        System.err.println("ERROR: directory "+directory+" not found");
                        continue;
                    }
                    ModuleRepository repository
                        = new PathModuleRepository(directory);
                    leanEdit.installRepository(repository);
                } else if (args[paramIndex].equals("-l") ||
                           args[paramIndex].equals("--loadmodule")) {
                    // load a single module from a JAR file
                    paramIndex++;
                    if (paramIndex>=args.length) {
                        System.err.println("missing filename after --loadmodule");
                        System.exit(1);
                    }
                    File file = new File(args[paramIndex++]);
                    if (!file.isFile()) {
                        System.err.println("ERROR: file "+file+" not found");
                        continue;
                    }
                    if (!file.canRead()) {
                        System.err.println("ERROR: cannot read file "+file);
                        continue;
                    }
                    ResourceLoader resourceLoader
                        = new ZipResourceLoader(file);
                    leanEdit.installModule(resourceLoader);
                } else {
                    break;
                }
            }

            // create session
            LeanEditSession session = new LeanEditSession(leanEdit);

            // open files according to command line parameters
            boolean windowOpen = false;

        PARAMETER:
            while (paramIndex<args.length) {

                if (args[paramIndex].equals("--openmodule")) {
                    // open a special frame of a module

                    paramIndex++;
                    if (paramIndex>=args.length) {
                        System.err.println("ERROR: module name missing after --openmodule");
                        continue PARAMETER;
                    }

                    String moduleName = args[paramIndex];
                    String parameter = null;
                    paramIndex++;

                    int colonIndex = moduleName.indexOf(':');
                    if (colonIndex!=-1) {
                        parameter = moduleName.substring(colonIndex+1);
                        moduleName = moduleName.substring(0, colonIndex);
                    }

                    EditorModule module = leanEdit.getModuleManager().getModule(moduleName);
                    if (module==null) {
                        System.err.println("ERROR: module '"+moduleName+"' not installed");
                        continue PARAMETER;
                    }

                    if (module.open(session, parameter))
                        windowOpen = true;

                } else {
                    // open a file

                    File file = new File(args[paramIndex]);

                    if (session.open(file))
                        windowOpen = true;

                    paramIndex++;
                }
            }

            // create a frame if none is open yet
            if (!windowOpen)
                session.createNewFrame();

        } catch (Exception e) {
            Debug.report(e);
        }
    }
}
