/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: AbstractEditorApplication.java,v 1.24 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit;

import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.net.URL;
import java.lang.reflect.InvocationTargetException;
import org.xml.sax.SAXException;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.w3c.dom.Element;
import make.util.Debug;
import make.xml.DOMUtil;
import make.xml.SchemaManager;
import make.xml.DefaultSchemaManager;
import make.module.ResourceLoader;
import make.module.ModuleRepository;
import org.leanedit.module.EditorModule;
import org.leanedit.module.DefaultEditorModule;
import org.leanedit.module.BaseEditorModule;
import org.leanedit.module.ModuleManager;
import org.leanedit.plugins.PluginManager;
import org.leanedit.convert.ConverterManager;
import org.leanedit.convert.DefaultConverterManager;

public abstract class AbstractEditorApplication
    implements EditorApplication, EntityResolver,
	       ModuleManager {
    /*properties*************************************/
    private Vector sessions = new Vector();
    private PluginManager pluginManager = new PluginManager();
    private SchemaManager schemaManager = new DefaultSchemaManager();
    private ConverterManager converterManager = new DefaultConverterManager();
    private Hashtable modules = new Hashtable();
    protected EditorModule baseModule;
    /*Constructor************************************/
    protected AbstractEditorApplication() {
	try {
	    baseModule = new BaseEditorModule(this);
	    baseModule.install(this);
	} catch (Exception e) {
	    Debug.report(e, "while installing base module");
	}
    }
    /*Module (un)install*********************/
    protected void installModule(EditorModule module, EditorFrame frame) {
	module.install(frame);
    }
    protected void uninstallModule(EditorModule module, EditorFrame frame) {
	module.uninstall(frame);
    }
    protected void installModule(EditorModule module, EditorSession session) {
	module.install(session);
	for (Enumeration enum=session.getFrames(); enum.hasMoreElements();) {
	    Object frame = enum.nextElement();
	    if (frame instanceof EditorFrame) {
		EditorFrame editorFrame = (EditorFrame)frame;
		installModule(module, editorFrame);
	    }
	}
    }
    protected void uninstallModule(EditorModule module, EditorSession session) {
	for (Enumeration enum=session.getFrames(); enum.hasMoreElements();) {
	    Object frame = enum.nextElement();
	    if (frame instanceof EditorFrame) {
		EditorFrame editorFrame = (EditorFrame)frame;
		uninstallModule(module, editorFrame);
	    }
	}
	module.uninstall(session);
    }
    /*ModuleManager*******************************/
    public Enumeration getModuleNames() {
	return modules.keys();
    }
    public EditorModule getModule(String name) {
	return (EditorModule)modules.get(name);
    }
    public void installModule(EditorModule module) {
	String name = module.getName();
	Object oldValue = (EditorModule)modules.put(name, module);
	if (oldValue!=null) {
	    modules.put(name, oldValue);
	    System.err.println("WARNING: aborted dupe installation of "+
			       name);
	    return;
	}
	try {
	    module.install(this);
	    for (Enumeration enum=getSessions(); enum.hasMoreElements();) {
		EditorSession session = (EditorSession)enum.nextElement();
		installModule(module, session);
	    }
	} catch (RuntimeException e) {
	    modules.remove(name);
	    throw e;
	} catch (Error e) {
	    modules.remove(name);
	    throw e;
	}
    }
    public void uninstallModule(EditorModule module) {
	String name = module.getName();
	Object oldModule = modules.remove(name);
	if (oldModule!=module) {
	    System.err.println("ERROR: aborted invalid deinstallation "+
			       name);
	    return;
	}
	try {
	    for (Enumeration enum=getSessions(); enum.hasMoreElements();) {
		EditorSession session = (EditorSession)enum.nextElement();
		uninstallModule(module, session);
	    }
	    module.uninstall(this);
	} catch (RuntimeException e) {
	    Debug.report(e, "while trying to uninstall module "+name);
	}
    }
    /*Modules internal************************/
    protected void loadModule(String className) 
	throws ClassNotFoundException, NoSuchMethodException,
	       InstantiationException, IllegalAccessException,
	       InvocationTargetException {
	Class cls = Class.forName(className);
	EditorModule module
	    = (EditorModule)cls.newInstance();
	installModule(module);
    }
    protected void installModule(ResourceLoader rl) 
	throws IOException, SAXException, ClassNotFoundException {
	installModule(new DefaultEditorModule(this, rl));
    }
    protected void installRepository(ModuleRepository repository) {
	for (Enumeration enum=repository.getModuleNames();
	     enum.hasMoreElements();) {
	    String name = (String)enum.nextElement();
	    ResourceLoader rl = repository.getModule(name);
	    try {
		installModule(rl);
	    } catch (Exception e) {
		Debug.report(e, "while loading module "+name);
	    }
	}
    }
    /*EditorApplication***************************************/
    public Enumeration getSessions() {
	return sessions.elements();
    }
    public void addSession(EditorSession session) {
	sessions.addElement(session);
	for (Enumeration enum=modules.elements(); enum.hasMoreElements();) {
	    EditorModule module = (EditorModule)enum.nextElement();
	    installModule(module, session);
	}
    }
    public void removeSession(EditorSession session) {
	for (Enumeration enum=modules.elements(); enum.hasMoreElements();) {
	    EditorModule module = (EditorModule)enum.nextElement();
	    uninstallModule(module, session);
	}
	sessions.removeElement(session);
    }
    public PluginManager getPluginManager() {
	return pluginManager;
    }
    public SchemaManager getSchemaManager() {
	return schemaManager;
    }
    public ModuleManager getModuleManager() {
	return this;
    }
    public ConverterManager getConverterManager() {
	return converterManager;
    }
    public EntityResolver getEntityResolver() {
	return this;
    }
    /*EntityResolver********************************/
    public InputSource resolveEntity(String publicId,
				     String systemId)
	throws SAXException, IOException {
	try {
	    URL url = new URL(systemId);
	    String fileName = url.getFile();
	    InputStream is = getClass().getResourceAsStream(fileName);
	    if (is==null) {
		StringTokenizer st = new StringTokenizer(url.getFile(), "/");
		String file = null;
		while (st.hasMoreTokens())
		    file = st.nextToken();
		if (file==null)
		    return DOMUtil.emptyEntityResolver
			.resolveEntity(publicId, systemId);
		is = getClass().getResourceAsStream("/entities/"+file);
		if (is==null)
		    return DOMUtil.emptyEntityResolver
			.resolveEntity(publicId, systemId);
	    }
	    return new InputSource(is);
	} catch (IOException e) {
	}
	return DOMUtil.emptyEntityResolver
	    .resolveEntity(publicId, systemId);
    }
}
