/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: AbstractEditorSession.java,v 1.18 2001/12/22 12:14:58 max Exp $
 *
 */

package org.leanedit;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Locale;
import java.io.IOException;
import java.awt.Toolkit;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.Clipboard;
import javax.swing.Action;
import javax.swing.AbstractAction;
import org.xml.sax.SAXException;
import make.util.Debug;
import make.util.ArrayEnumeration;
import make.util.AppendEnumeration;
import make.swing.SwingSession;
import make.swing.ProxyClipboard;
import make.swing.progress.ProgressIndicatorFactory;
import make.swing.panel.InterfacePanel;
import make.swing.panel.DialogInterfacePanel;
import make.swing.panel.PanelContent;
import make.swing.panel.ProxyInterfacePanel;
import make.swing.panel.ProxyPanelContent;
import make.datamodel.embedded.EmbeddedManager;
import org.leanedit.module.EditorModule;
import org.leanedit.module.ModuleManager;
import org.leanedit.control.ControlPanel;

public abstract class AbstractEditorSession implements EditorSession {
    private class EventPanelContent extends ProxyPanelContent {
        public EventPanelContent(PanelContent _pc) {
            super(_pc);
            addFrame(_pc);
        }
        public void panelClosed() {
            removeFrame(parent);
            super.panelClosed();
        }
    }
    private class EventInterfacePanel extends ProxyInterfacePanel {
        public EventInterfacePanel(InterfacePanel _ip) {
            super(_ip);
        }
        public void setContent(PanelContent c) {
            super.setContent(c==null ? null : new EventPanelContent(c));
        }
    }
    /*properties******************************/
    private Vector frames = new Vector();
    private boolean inClose = false;
    protected EditorApplication application;
    protected final Toolkit toolkit = Toolkit.getDefaultToolkit();
    protected Clipboard clipboard;
    private Action[] myActions;
    private Vector additionalActions;
    /*Constructor*********************************/
    public AbstractEditorSession(EditorApplication _app) {
        try {
            clipboard = new ProxyClipboard(toolkit.getSystemClipboard());
        } catch (SecurityException e) {
            // create an internal clipboard if the JVM
            // doesn't allow access to the system clipboard
            clipboard = new Clipboard("[simpleClipboard]");
        }
        application = _app;
        application.addSession(this);
    }
    protected void finalize() 
        throws Throwable {
        if (Debug.DEBUG && !frames.isEmpty()) {
            System.err.println("WARNING: in AbstractEditorSession.finalize, "+
                               "but !frames.isEmpty()");
        }
        super.finalize();
    }
    /*AbstractEditorSession public************************************/
    public abstract EditorFrame createNewFrame(InterfacePanel ip);
    public void createNewFrame() {
        InterfacePanel ip = createInterfacePanel();
        createNewFrame(ip);
        ip.setVisible(true);
    }
    /*AbstractEditorSession protected***********************************/
    protected void setSessionActions(Action[] _sa) {
        myActions = _sa;
    }
    protected void addFrame(PanelContent frame) {
        frames.addElement(frame);
        if (frame instanceof EditorFrame) {
            // tell all modules about our new frame
            EditorFrame editorFrame = (EditorFrame)frame;
            ModuleManager moduleManager = application.getModuleManager();
            for (Enumeration enum=moduleManager.getModuleNames(); 
                 enum.hasMoreElements();) {
                String name = (String)enum.nextElement();
                EditorModule module = moduleManager.getModule(name);
                module.install(editorFrame);
            }
        }
    }
    protected void removeFrame(PanelContent frame) {
        if (frame instanceof EditorFrame) {
            // notify all modules about the closed frame
            EditorFrame editorFrame = (EditorFrame)frame;
            ModuleManager moduleManager = application.getModuleManager();
            for (Enumeration enum=moduleManager.getModuleNames(); 
                 enum.hasMoreElements();) {
                String name = (String)enum.nextElement();
                EditorModule module = moduleManager.getModule(name);
                module.uninstall(editorFrame);
            }
        }
        frames.removeElement(frame);
    }
    protected InterfacePanel wrapInterfacePanel(InterfacePanel ip) {
        return new EventInterfacePanel(ip);
    }
    /*SwingSession***********************************************/
    public Locale getLocale() {
        return Locale.getDefault();
    }
    public Enumeration getFrames() {
        return frames.elements();
    }
    public Enumeration getActions() {
        Enumeration enum = new ArrayEnumeration(defaultActions);
        if (myActions!=null)
            enum = new AppendEnumeration(enum, new ArrayEnumeration(myActions));
        if (additionalActions!=null)
            enum = new AppendEnumeration(enum, additionalActions.elements());
        return enum;
    }
    public abstract InterfacePanel createInterfacePanel();
    public Clipboard getClipboard() {
        return clipboard;
    }

    /*EditorSession***********************************************/
    public EditorApplication getApplication() { 
        return application; 
    }
    public abstract ProgressIndicatorFactory getProgressIndicatorFactory();
    public abstract EditorFrame open(EmbeddedManager em, InterfacePanel ip)
        throws IOException, SAXException;
    public void addActions(Enumeration enum) {
        if (additionalActions==null)
            additionalActions = new Vector();
        while (enum.hasMoreElements())
            additionalActions.addElement((Action)enum.nextElement());
    }
    public void removeActions(Enumeration enum) {
        while (enum.hasMoreElements())
            additionalActions.removeElement((Action)enum.nextElement());
    }
    public boolean close(boolean interactive) {
        if (inClose)
            return false;
        try {
            inClose = true;
            for (Enumeration enum=frames.elements(); enum.hasMoreElements();) {
                PanelContent panelContent = (PanelContent)enum.nextElement();
                if (panelContent instanceof EditorFrame) {
                    EditorFrame editorFrame = (EditorFrame)panelContent;
                    if (!editorFrame.close(interactive))
                        return false;
                }
            }
            application.removeSession(this);
            return true;
        } finally {
            inClose = false;
        }
    }
    /*Actions*****************************************/
    private Action[] defaultActions = {
        new AboutAction(),
        new ControlPanelAction(),
    };
    class AboutAction extends AbstractAction {
        AboutAction() { super("about"); }
        public void actionPerformed(ActionEvent e) {
            InterfacePanel ip = new DialogInterfacePanel(null);
            new AboutDialog(ip);
            ip.setVisible(true);
        }
    }
    class ControlPanelAction extends AbstractAction {
        ControlPanelAction() { super("control-panel"); }
        public void actionPerformed(ActionEvent evt) {
            InterfacePanel interfacePanel = createInterfacePanel();
            ControlPanel controlPanel = new ControlPanel
                (AbstractEditorSession.this, interfacePanel);
            interfacePanel.setContent(controlPanel);
            interfacePanel.setVisible(true);
        }
    }
}
