/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorDocument.java,v 1.9 2002/03/12 21:19:08 max Exp $
 *
 */

package org.leanedit;

import java.util.Vector;
import java.util.Enumeration;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import javax.swing.JOptionPane;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.SAXException;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPath;
import org.apache.xpath.objects.XObject;
import make.datamodel.embedded.EmbeddedManager;
import make.util.EmptyEnumeration;
import make.util.ArrayEnumeration;
import make.xml.XPathUtil;
import make.xml.DOMUtil;

public final class EditorDocument {
    /*properties*******************************/
    protected Node node;
    protected EditorDocumentType type;
    protected EmbeddedManager embeddedManager;
    protected EditorDocument parent;
    protected String name;
    /*Constructor******************************/
    public EditorDocument(Node _el, EditorDocumentType _dt) {
        node = _el; type = _dt;
    }
    public EditorDocument(Node _el, EditorDocumentType _dt,
                          EmbeddedManager _em) {
        this(_el, _dt);
        embeddedManager = _em;
    }
    public EditorDocument(EditorDocumentType _dt,
                          EmbeddedManager _em) 
        throws IOException, SAXException {
        type = _dt;
        embeddedManager = _em;
        node = embeddedManager.getDocument();
    }
    public EditorDocument(Node _el, EditorDocumentType _dt,
                          EditorDocument _parent) {
        this(_el, _dt);
        parent = _parent;
    }
    public EditorDocument(Node _el, EditorDocument _parent) {
        this(_el, _parent.getType(), _parent);
    }
    /*IO*************************************/
    public void writeDocument(EmbeddedManager em) throws IOException {
        em.replaceDocument(node);
    }
    public void writeDocument() throws IOException {
        writeDocument(getEmbeddedManager());
    }
    /*Info*************************************/
    public Enumeration getEmbeddedNames() {
        return type==null
            ?(Enumeration)new EmptyEnumeration()
            :(Enumeration)new ArrayEnumeration(type.getEmbeddedNames(node));
    }
    public void copyEmbedded(EmbeddedManager src, EmbeddedManager dest) {
        for (Enumeration enum=getEmbeddedNames(); enum.hasMoreElements();) {
            String name = (String)enum.nextElement();
            if (name.equals("_dummy")) continue;
            InputStream is = null;
            try {
                is = src.getEntry(name);
                if (is!=null)
                    dest.replaceEntry(name, is);
            } catch (FileNotFoundException e) {
            } catch (IOException e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog
                    (null, "Fehler beim Kopieren der eingebetteten Objekte: "+
                     e.getLocalizedMessage(),
                     "Fehler beim Kopieren", JOptionPane.ERROR_MESSAGE);
            } finally {
                try {
                    if (is!=null)
                        is.close();
                } catch (IOException e) {
                }
            }
        }
    }
    public void putNewEmbeddedManager(EmbeddedManager _em) {
        if (embeddedManager!=null)
            copyEmbedded(embeddedManager, _em);
        embeddedManager = _em;
    }
    public void setEmbeddedManager(EmbeddedManager _em) {
        embeddedManager = _em;
    }
    public XObject execute(String s) 
        throws TransformerException {
        if (type==null) {
            return XPathUtil.execute(s, node);
        } else {
            XPath pt = type.parseXPath(s);
            return XPathUtil.execute(pt, node);
        }
    }
    public EditorDocument[] followPath(String path, String name, 
                                       boolean create) 
        throws TransformerException {
        NodeIterator ni = execute(path).nodeset();
        Node node = ni.nextNode();
        EditorDocument[] docs;
        if (node==null && create) {
            docs = new EditorDocument[1];
            Element data = DOMUtil.createDeepUniqueTag((Element)getNode(), path);
            docs = new EditorDocument[] {
                new EditorDocument(data, this),
            };
            docs[0].setName(name);
        } else {
            Vector vc = new Vector();
            vc.addElement(node);
            while ((node=ni.nextNode())!=null)
                vc.addElement(node);
            docs = new EditorDocument[vc.size()];
            for (int z=0; z<docs.length; z++) {
                docs[z] = new EditorDocument((Element)vc.elementAt(z), this);
                docs[z].setName(name);
            }
        }
        return docs;
    }
    /*EditorDocument*****************************/
    public Node getNode() { return node; }
    public Document getXMLDocument() {
        return node.getOwnerDocument();
    }
    public EditorDocumentType getType() { return type; }
    public EmbeddedManager getEmbeddedManager() {
        return parent==null?embeddedManager:parent.getEmbeddedManager();
    }
    public EditorDocument getParent() { return parent; }
    public String getName() {
        if (name!=null)
            return name;
        return node.getNodeName();
    }
    public void setName(String _n) {
        name = _n;
    }
}
