/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorDocumentType.java,v 1.11 2002/03/12 20:15:30 max Exp $
 *
 */

package org.leanedit;

import java.io.InputStream;
import java.io.FileNotFoundException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Random;
import java.util.Date;
import java.text.DateFormat;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Attr;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import org.apache.xpath.XPathContext;
import org.apache.xpath.XPath;
import org.apache.xpath.objects.XObject;
import org.apache.xpath.objects.XNumber;
import org.apache.xpath.objects.XString;
import org.apache.xpath.functions.Function;
import org.apache.xpath.compiler.FunctionTable;
import make.io.Streams;
import make.xml.DocumentType;
import make.xml.DOMUtil;
import make.xml.ConfigPath;
import make.util.Debug;
import make.util.Strings;

public class EditorDocumentType extends DocumentType {
    /***properties******************************/
    protected EditorDocumentView defaultView;
    protected Hashtable views = new Hashtable();
    protected EditorApplication application;
    protected TransformerFactory tFactory = TransformerFactory.newInstance();
    /***Constructor********************************/
    public EditorDocumentType(EditorApplication _app, ConfigPath _cp) 
        throws FileNotFoundException {
        super(_cp);
        application = _app;
        // views
        Element v = getConfigPath().parse("views.xml");
        if (v==null)
            v = getMisc("views");
        loadViews(v);
        // lists
        Element lists = getConfigPath().parse("lists.xml");
        if (lists!=null)
            loadLists(lists);
    }
    protected void loadViews(Element pl) {
        if (pl==null) return;
        String defaultName = pl.getAttribute("default");
        Element[] vis = DOMUtil.getChildElements(pl, "view");
        for (int z=0; z<vis.length; z++) {
            EditorDocumentView view = new EditorDocumentView(this, vis[z]);
            String name = view.getName();
            views.put(name, view);
            if (name.equals(defaultName))
                defaultView = view;
        }
    }
    /***Info*******************************/
    public EditorDocumentView getView(String name) {
        return (EditorDocumentView)views.get(name);
    }
    public EditorDocumentView getDefaultView() { 
        return defaultView;
    }
    public Enumeration getViews() { return views.elements(); }
    public EditorApplication getApplication() { return application; }
    /***DocumentType*****************************/
    public URL getResource(String uri) {
        return getClass().getResource("/formats/"+name+'/'+uri);
    }
    /***Functions*******************************/
    public static class FuncRandomInt32 extends Function {
        Random random = new Random();
        public XObject execute(XPathContext execContext) 
            throws TransformerException {
            return new XNumber(random.nextInt());
        }
        public void fixupVariables(Vector vars, int globalsSize) {}
    }
    public static class FuncNow extends Function {
        DateFormat format = Strings.storeDateFormats[Strings.DATETIME];
        public XObject execute(XPathContext execContext) 
            throws TransformerException {
            return new XString(format.format(new Date()));
        }
        public void fixupVariables(Vector vars, int globalsSize) {}
    }
    static {
        FunctionTable.installFunction("random-int32", new FuncRandomInt32());
        FunctionTable.installFunction("now", new FuncNow());
    }
    /***Misc.***********************************/
    public Document createNewDocument(EditorSession session) {
        Document result = null;
        try {
            URL url = getResource("new.xml");
            if (tFactory!=null && url!=null) {
                InputStream is = url.openStream();
                Document xslDoc = DOMUtil.parse(is, Streams.defaultEncoding);
                Element xslRoot = xslDoc.getDocumentElement();
                if (xslRoot.getAttribute("xmlns:xsl")!=null &&
                    xslRoot.getAttribute("xmlns:xsl").indexOf("w3")>=0) {
                    Transformer transformer
                        = tFactory.newTransformer(new DOMSource(xslDoc));
                    Document xmlDoc = DOMUtil.createDocument();
                    xmlDoc.appendChild(xmlDoc.createElement("null"));
                    result = DOMUtil.createDocument();
                    transformer.transform(new DOMSource(xmlDoc), 
                                          new DOMResult(result));
                } else {
                    return xslDoc;
                }
            }
        } catch (Exception e) {
            Debug.report(e);
            result = null;
        }
        if (result==null) {
            result = DOMUtil.createDocument();
            result.appendChild(result.createElement(name));
        }
        return result;
    }
}
