/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorPanel.java,v 1.8 2001/10/12 20:03:13 max Exp $
 *
 */

package org.leanedit;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.text.ParseException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import org.w3c.dom.Element;
import make.util.Debug;
import make.swing.panel.InterfacePanel;
import make.swing.panel.PanelContent;
import org.leanedit.plugins.EditorPlugin;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.ComponentListener;
import org.leanedit.plugins.ComponentEvent;
import org.leanedit.plugins.CompositeComponent;

/**
 * EditorPanel is an org.leanedit.EditorFrame implementation
 * which can display a DOM tree and lets the user edit it.
 * It provides no further interface to the user what to do
 * with the document.
 */
public abstract class EditorPanel extends DefaultPanel 
    implements EditorFrame, PropertyChangeListener,
	       ComponentListener {
    /*static**************************/
    protected static final ResourceBundle resources
	= ResourceBundle.getBundle("org.leanedit.EditorFrame");
    /*properties*************************/
    private JPanel panel;
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private EditorDocument document;
    private EditorDocumentView view;
    private Component installedComponent;
    private boolean inClose = false;
    protected EditorComponent editorComponent;
    protected boolean changed = false, flushRequired = false;
    protected EditorPlugin plugin;
    protected Hashtable components = new Hashtable();
    /*Constructor*****************************/
    public EditorPanel(EditorSession _sess, InterfacePanel _ip) {
	super(_sess, new JPanel(new BorderLayout()), _ip);
	panel = (JPanel)getComponent();
    }
    /*View*********************************/
    protected void setEditorPlugin(EditorPlugin _plugin, EditorDocumentView _v) {
	if (plugin==_plugin && view==_v) return;
	final EditorComponent oldComponent = editorComponent;
	// remove old component
	if (editorComponent!=null) {
	    if (view!=null) {
		EditorDocumentView oldView = view;
		view = null;
		firePropertyChange(VIEW, oldView, view);
		oldView = view;
	    }
	    // check if there are parse errors
	    if (_plugin!=null && document!=null &&
		getDocument()==null)
		return;
	    editorComponent.removeComponentListener(this);
	    editorComponent.removePropertyChangeListener(this);
	    panel.remove(installedComponent);
	    installedComponent = null;
	    editorComponent.dispose();
	    editorComponent = null;
	}
	// some debug code
	if (Debug.DEBUG && !components.isEmpty()) {
	    System.err.println("some unregistered components:");
	    for (Enumeration enum=components.elements(); 
		 enum.hasMoreElements();)
		System.err.println(enum.nextElement());
	    throw new InternalError();
	}
	// install new plugin
	plugin = _plugin;
	// create new component
	if (plugin!=null && document!=null) {
	    view = _v;
	    // component
	    editorComponent = plugin.createComponent(this);
	    editorComponent.setDocument(document);
	    editorComponent.focusGained();
	    installedComponent = editorComponent.getComponent();
	    if (installedComponent!=null)
		panel.add("Center", installedComponent);
	    editorComponent.addPropertyChangeListener(this);
	    editorComponent.addComponentListener(this);
	} else {
	    view = null;
	}
	panel.validate();
	panel.repaint();
	// fire
	if (editorComponent!=oldComponent)
	    firePropertyChange(COMPONENT, oldComponent, editorComponent);
	if (view!=null)
	    firePropertyChange(VIEW, null, view);
    }
    protected void setView(EditorDocumentView _vn) {
	if (_vn==null)
	    setEditorPlugin(null, null);
	else if (_vn!=view) {
	    EditorDocumentType edt = document.getType();
	    EditorPlugin pl = _vn.loadPlugin(); 
	    setEditorPlugin(pl, _vn);
	}
    }
    public void setDocument(EditorDocument _doc, EditorPlugin _plugin, 
			    EditorDocumentView _v) {
	EditorDocument oldDocument = document;
	setEditorPlugin(null, null);
	document = _doc;
	firePropertyChange(DOCUMENT, oldDocument, document);
	try {
	    setEditorPlugin(_plugin, _v);
	} catch (Exception e) {
	    Debug.report(e, "during component creation");
	    setEditorPlugin(null, null);
	}
    }
    public void setDocument(EditorDocument _doc) {
	if (_doc==null) {
	    setDocument(null, null, null);
	    getInterfacePanel().setTitle(null);
	} else {
	    EditorDocumentType edt = _doc.getType();
	    EditorDocumentView view = edt==null
		?new EditorDocumentView(getApplication().getPluginManager())
		    :edt.getDefaultView();
	    EditorPlugin plugin = view.loadPlugin();
	    setDocument(_doc, plugin, view);
	    getInterfacePanel().setTitle(_doc.getEmbeddedManager().toString());
	}
    }
    /*Util********************************/
    protected void firePropertyChange(String name, Object o, Object n) {
	pcs.firePropertyChange(name, o, n);
    }
    /*Info**********************************/
    public final EditorApplication getApplication() { 
	return getSession().getApplication(); 
    }
    /*EditorPanel******************************/
    protected boolean documentClosing() {
	return true;
    }
    /*EditorFrame********************************/
    public final EditorSession getSession() { 
	return (EditorSession)super.getSwingSession(); 
    }
    public EditorDocumentView getView() { 
	return view; 
    }
    public void registerComponent(String key, CompositeComponent ec) {
	Object ov = components.put(key, ec);
	if (Debug.DEBUG && ov!=null)
	    throw new InternalError();
    }
    public void unregisterComponent(String key, CompositeComponent ec) {
	Object ov = components.remove(key);
	if (Debug.DEBUG) {
	    if (ov!=ec)
		throw new InternalError();
	}
    }
    public EditorComponent[] getComponents(String key) {
	if (COMPONENT_ROOT.equals(key)) {
	    if (editorComponent==null)
		return null;
	    else
		return new EditorComponent[]{editorComponent};
	}
	CompositeComponent cc = (CompositeComponent)components.get(key);
	if (cc==null)
	    return null;
	return cc.getComponents(key);
    }
    public EditorDocument getDocument() {
	if (document==null)
	    return null;
	try {
	    if (flushRequired)
		editorComponent.flushDocument();
	    return document;
	} catch (ParseException e) {
	    JOptionPane.showMessageDialog
		(getFrame(), e.getLocalizedMessage(),
		 resources.getString("error.input"),
		 JOptionPane.ERROR_MESSAGE);
	    return null;
	}
    }
    public void addPropertyChangeListener(PropertyChangeListener l) {
	pcs.addPropertyChangeListener(l);
    }
    public void removePropertyChangeListener(PropertyChangeListener l) {
	pcs.removePropertyChangeListener(l);
    }
    public boolean close(boolean interactive) {
	if (inClose)
	    return false;
	try {
	    inClose = true;
	    if (interactive && !documentClosing())
		return false;
	    return true;
	} finally {
	    inClose = false;
	}
    }
    /*PanelContent**************************/
    public boolean panelClosing() {
	return super.panelClosing() && documentClosing();
    }
    public void panelClosed() {
	setDocument(null);
	if (editorComponent!=null) {
	    editorComponent.dispose();
	    editorComponent = null;
	}
	super.panelClosed();
    }
    /*ComponentListener****************************/
    public void focusGained(ComponentEvent evt) {}
    public void documentChanged(ComponentEvent evt) {
	changed = true; flushRequired = true;
    }
    /*PropertChangeListener**********************/
    public void propertyChange(PropertyChangeEvent evt) {
	if (evt.getSource()==editorComponent &&
	    evt.getPropertyName().equals(editorComponent.COMPONENT)) {
	    if (installedComponent!=null)
		panel.remove(installedComponent);
	    installedComponent = (Component)evt.getNewValue();
	    if (installedComponent!=null)
		panel.add("Center", installedComponent);
	    panel.validate();
	}
    }
    /*Actions********************************/
    public abstract class DocumentAction extends AbstractAction {
	/*Constructor******/
	public DocumentAction(String ac) {
	    super(ac); 
	    setEnabled(false); 
	}
	/*AbstractAction*****/
	public void actionPerformed(ActionEvent evt) {
	    EditorDocument doc = getDocument();
	    if (doc!=null)
		actionPerformed(evt, doc);
	}
	/*DocumentAction********/
	public final void update() {
	    update(document);
	}
	public void update(EditorDocument doc) {
	    setEnabled(doc!=null);
	}
	public abstract void actionPerformed(ActionEvent evt, 
					     EditorDocument document);
    }
}
