/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: LeanEditSession.java,v 1.11 2001/12/22 12:14:58 max Exp $
 *
 */

package org.leanedit;

import java.util.Vector;
import java.io.IOException;
import java.io.File;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;
import org.xml.sax.SAXException;
import make.util.ErrorUtil;
import make.swing.ThreadAction;
import make.swing.bt.FrameDescription;
import make.swing.bt.FrameDescriptionCache;
import make.swing.progress.ProgressIndicatorFactory;
import make.swing.progress.FrameProgressIndicatorFactory;
import make.swing.panel.InterfacePanel;
import make.swing.panel.PanelContent;
import make.swing.panel.FrameInterfacePanel;
import make.datamodel.embedded.EmbeddedManager;

public class LeanEditSession extends AbstractEditorSession {
    /*static**********************************/
    protected static final EditorFileChooser fileChooser
        = new EditorFileChooser();
    /*properties*************************/
    protected FrameDescription frameDescription;
    protected ProgressIndicatorFactory progressIndicatorFactory
        = new FrameProgressIndicatorFactory();
    private Vector installedFrames = new Vector();
    /*Constructor*********************************/
    public LeanEditSession(EditorApplication _app) {
        super(_app);
        setSessionActions(defaultActions);
        frameDescription = FrameDescriptionCache.getFrameDescription
            (LeanEditSession.class, "frame.xml", getLocale());
    }
    /*AbstractEditorSession public************************************/
    public EditorFrame createNewFrame(InterfacePanel ip) {
        EditorFrame frame = new SingleEditorFrame(this, ip);
        if (frameDescription!=null)
            ip.addFrameDescription(frameDescription);
        installedFrames.addElement(frame);
        ip.setContent(frame);
        return frame;
    }
    /*AbstractEditorSession protected**********************************/
    protected void removeFrame(PanelContent frame) {
        // remove FrameDescription
        if (installedFrames.contains(frame)) {
            EditorFrame editorFrame = (EditorFrame)frame;
            InterfacePanel interfacePanel = editorFrame.getInterfacePanel();
            if (frameDescription!=null)
                interfacePanel.removeFrameDescription(frameDescription);
            installedFrames.remove(frame);
        }
        // call super class
        super.removeFrame(frame);
        // close the session if no frames are left
        if (!getFrames().hasMoreElements())
            close(false);
    }
    /*SwingSession*************************************/
    public InterfacePanel createInterfacePanel() {
        return wrapInterfacePanel(new FrameInterfacePanel());
    }
    public ProgressIndicatorFactory getProgressIndicatorFactory() {
        return progressIndicatorFactory;
    }
    /*EditorSession**********************************/
    public EditorFrame open(EmbeddedManager em, InterfacePanel ip)
        throws IOException, SAXException {
        if (ip==null)
            ip = createInterfacePanel();
        EditorFrame frame = createNewFrame(ip);
        ((TopEditorFrame)frame).readFrom(em);
        ip.setVisible(true);
        return frame;
    }
    /*Actions*****************************************/
    private Action[] defaultActions = {
        new ThreadAction(new NewFrameAction()),
        new ThreadAction(new OpenAction()),
        new GarbageCollectAction(),
    };
    class NewFrameAction extends AbstractAction {
        NewFrameAction() { super("newframe"); }
        public void actionPerformed(ActionEvent e) {
            createNewFrame();
        }
    }
    public boolean open(File f) {
        try {
            if (!f.exists()) {
                JOptionPane.showMessageDialog
                    (null, "Die Datei "+f.getAbsolutePath()+" existiert nicht.",
                     "Fehler", JOptionPane.ERROR_MESSAGE, null);
                return false;
            }
            if (f.isDirectory()) {
                JOptionPane.showMessageDialog
                    (null, "Die Datei "+f.getAbsolutePath()+" ist ein Verzeichnis.",
                     "Fehler", JOptionPane.ERROR_MESSAGE, null);
                return false;
            }
            if (!f.canRead()) {
                JOptionPane.showMessageDialog
                    (null, "Zugriff auf die Datei "+f.getAbsolutePath()+" verweigert.",
                     "Fehler", JOptionPane.ERROR_MESSAGE, null);
                return false;
            }
        } catch (SecurityException e) {
            JOptionPane.showMessageDialog
                (null, "Zugriff auf die Datei "+f.getAbsolutePath()+" verweigert.",
                 "Fehler", JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        try {
            InterfacePanel ip = createInterfacePanel();
            EditorFrame frame = createNewFrame(ip);
            ((TopEditorFrame)frame).readFrom(f);
            ip.setVisible(true);
            return true;
        } catch (Exception ee) {
            JOptionPane.showMessageDialog
                (null, ErrorUtil.format(ee),
                 "Fehler beim Laden", JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
    }
    class OpenAction extends AbstractAction {
        OpenAction() { super("open-file"); }
        public void actionPerformed(ActionEvent e) {
            fileChooser.getUI().rescanCurrentDirectory(fileChooser);
            final int chooseRet = fileChooser.showOpenDialog(null);
            if (chooseRet!=fileChooser.APPROVE_OPTION)
                return;
            File f = fileChooser.getSelectedFile();
            if (f!=null) open(f);
        }
    }
    class GarbageCollectAction extends AbstractAction {
        GarbageCollectAction() { super("garbage-collect"); }
        public void actionPerformed(ActionEvent evt) {
            System.gc();
            System.runFinalization();
        }
    }
}
