/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2002 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SingleEditorFrame.java,v 1.22 2002/03/14 19:16:16 max Exp $
 *
 */

package org.leanedit;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Iterator;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.File;
import java.io.Reader;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import org.leanedit.plugins.EditorPlugin;
import make.util.Debug;
import make.util.ErrorUtil;
import make.io.Streams;
import make.mime.Message;
import make.mime.FileMessage;
import make.xml.DOMUtil;
import make.xml.SchemaManager;
import make.xml.DocumentType;
import make.swing.ButtonManager;
import make.swing.panel.InterfacePanel;
import make.datamodel.embedded.EmbeddedManager;
import make.datamodel.embedded.FileEmbeddedManager;
import make.datamodel.embedded.DocumentReplaceEmbeddedManager;
import org.leanedit.embedded.NewEmbeddedManager;
import org.leanedit.embedded.TransformEmbeddedManager;
import org.leanedit.convert.Converter;
import org.leanedit.convert.ConverterLoader;
import org.leanedit.convert.ConverterException;
import org.leanedit.convert.ConverterChooser;
import org.leanedit.schema.SchemaChooser;

/**
 * SingleEditorFrame is an EditorFrame implementation that acts
 * as user interface in Single Document Interface (SDI) applications.
 * It provides operations to the user like open a new file,
 * close the current file, import or export file. All of these
 * operations concentrate on the fact that this object is reusable
 * for the whole lifetime of the session.
 */
public class SingleEditorFrame extends TopEditorFrame {
    /*static**********************************/
    protected static EditorFileChooser fileChooser
        = new EditorFileChooser();
    protected static JFileChooser converterFileChooser
        = new JFileChooser();
    /*properties******************************/
    protected Action[] newActions;
    /*Constructor*****************************/
    public SingleEditorFrame(EditorSession _sess, InterfacePanel _ip) {
        super(_sess, _ip);
        // default actions
        getInterfacePanel().getButtonManager().installActions(defaultActions);
        // new-actions
        Vector na = new Vector();
        SchemaManager fm = getApplication().getSchemaManager();
        for (Enumeration enum=fm.getSchemaNames(); enum.hasMoreElements();) {
            na.addElement(new NewAction((String)enum.nextElement()));
        }
        newActions = new Action[na.size()];
        for (int z=0; z<na.size(); z++)
            newActions[z] = (Action)na.elementAt(z);
        getInterfacePanel().getButtonManager().installActions(newActions);
    }
    protected void finalize() throws Throwable {
        super.finalize();
    }
    /*EditorPanel*********************************/
    public void setDocument(EditorDocument _doc, EditorPlugin _plugin, 
                            EditorDocumentView _v) {
        super.setDocument(_doc, _plugin, _v);
        dumpAction.update();
        saveAsAction.update();
        exportAction.update();
        viewsAction.update();
        closeFileAction.update();
    }
    /*PanelContent**************************/
    public void panelClosed() {
        getInterfacePanel().getButtonManager().uninstallActions(newActions);
        getInterfacePanel().getButtonManager().uninstallActions(defaultActions);
        super.panelClosed();
    }
    /*File*****************************************/
    public boolean saveAs(EditorDocument doc) {
        fileChooser.getUI().rescanCurrentDirectory(fileChooser);
        final int chooseRet = fileChooser.showSaveDialog(getFrame());
        if (chooseRet!=fileChooser.APPROVE_OPTION)
            return false;
        File f = fileChooser.getSelectedFile();
        if (f.getName().indexOf('.')==-1) {
            if (fileChooser.getFileFilter() instanceof EditorFileChooser.XmlFileFilter)
                f = new File(f.getPath()+".xml");
            else
                f = new File(f.getPath()+".zip");
        }
        if (f.exists() && JOptionPane.showConfirmDialog
            (getFrame(), 
             resources.getString("ask.overwrite"),
             resources.getString("operation.saveAs"),
             JOptionPane.YES_NO_OPTION,
             JOptionPane.QUESTION_MESSAGE)!=JOptionPane.YES_OPTION)
            return false;
        try {
            if (f.getName().endsWith(".html") || f.getName().endsWith(".htm")) {
                // export to HTML
                EmbeddedManager em = new TransformEmbeddedManager
                    (getApplication().getSchemaManager(),
                     new FileEmbeddedManager(f, null, Streams.defaultEncoding), 
                     "html", null);
                doc.copyEmbedded(doc.getEmbeddedManager(), em);
                writeDocument(doc, em);
            } else if (f.getName().endsWith(".fo")) {
                // export to FO
                if (doc.getType()==null)
                    throw new IOException
                        (resources.getString("error.export.filter.notfound"));
                Node n = doc.getNode();
                Document xdoc = n.getOwnerDocument();
                // open XSLT
                InputStream xslIS = doc.getType()
                    .getConfigPath().open("export_fo.xsl");
                if (xslIS==null)
                    throw new IOException
                        (resources.getString("error.export.filter.notfound"));
                Reader xslReader = Streams.streamToReader(xslIS);
                // transform
                OutputStream os = new FileOutputStream(f);
                TransformerFactory tFactory = TransformerFactory.newInstance();
                Transformer transformer = tFactory.newTransformer
                    (new StreamSource(xslReader));
                transformer.transform
                    (new DOMSource(xdoc), new StreamResult(os));
                os.close();
            } else {
                // simply save into new file
                EmbeddedManager em = createEmbeddedManager(f, getApplication().getSchemaManager());
                saveAll(doc, em);
                getInterfacePanel().setTitle(em.toString());
            }
            return true;
        } catch (Exception ee) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(ee),
                 resources.getString("operation.saveAs"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
    }
    public boolean exportFile(EditorDocument doc) {
        // choose+configure a Converter object
        LinkedList converters = new LinkedList();
        for (Iterator it=getApplication().getConverterManager().getConverters();
             it.hasNext();) {
            ConverterLoader loader = (ConverterLoader)it.next();
            Converter converter = loader.getConverter();
            if (converter==null || 
                (converter.exportPossible() && converter.exportPossible(doc.getType())))
                converters.add(loader);
        }
        Hashtable config = new Hashtable();
        Converter converter = ConverterChooser.chooseConverter
            (getFrame(), converters.iterator(), config);
        if (converter==null)
            return false;
        if (!converter.exportPossible()) {
            JOptionPane.showMessageDialog
                (getFrame(),
                 "This converter is unable to export documents.",
                 resources.getString("operation.export"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        if (!converter.exportPossible(doc.getType())) {
            JOptionPane.showMessageDialog
                (getFrame(),
                 "This converter is unable to export this document.",
                 resources.getString("operation.export"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        // export in memory
        Message msg;
        try {
            // this trick is because we havnt saved the document yet,
            // so we simulate we had done it.
            EmbeddedManager embeddedManager
                = new DocumentReplaceEmbeddedManager
                (doc.getEmbeddedManager(), doc.getNode());
            msg = converter.exportDocument
                (embeddedManager, doc.getType(), config);
        } catch (ConverterException e) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(e),
                 resources.getString("operation.export"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        // choose a destination file
        converterFileChooser.getUI().rescanCurrentDirectory(converterFileChooser);
        final int chooseRet = converterFileChooser.showSaveDialog(getFrame());
        if (chooseRet!=converterFileChooser.APPROVE_OPTION)
            return false;
        File file = converterFileChooser.getSelectedFile();
        if (file==null)
            return false;
        if (file.exists() && JOptionPane.showConfirmDialog
            (getFrame(), 
             resources.getString("ask.overwrite"),
             resources.getString("operation.export"),
             JOptionPane.YES_NO_OPTION,
             JOptionPane.QUESTION_MESSAGE)!=JOptionPane.YES_OPTION)
            return false;
        // write the converted data to disk
        try {
            OutputStream os = new FileOutputStream(file);
            InputStream is = msg.getInputStream();
            Streams.copy(is, os);
            os.close();
        } catch (IOException e) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(e),
                 resources.getString("operation.export"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        return true;
    }
    public boolean importFile() {
        if (!documentClosing())
            return false;
        // choose+configure a Converter object
        LinkedList converters = new LinkedList();
        for (Iterator it=getApplication().getConverterManager().getConverters();
             it.hasNext();) {
            ConverterLoader loader = (ConverterLoader)it.next();
            Converter converter = loader.getConverter();
            if (converter==null || converter.importPossible())
                converters.add(loader);
        }
        Hashtable config = new Hashtable();
        Converter converter = ConverterChooser.chooseConverter
            (getFrame(), converters.iterator(), config);
        if (converter==null)
            return false;
        if (!converter.importPossible()) {
            JOptionPane.showMessageDialog
                (getFrame(),
                 "This converter is unable to import documents.",
                 resources.getString("operation.import"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        // choose a source file
        converterFileChooser.getUI().rescanCurrentDirectory(converterFileChooser);
        final int chooseRet = converterFileChooser.showOpenDialog(getFrame());
        if (chooseRet!=converterFileChooser.APPROVE_OPTION)
            return false;
        File file = converterFileChooser.getSelectedFile();
        if (file==null)
            return false;
        if (!file.exists()) {
            JOptionPane.showMessageDialog
                (getFrame(),
                 "The selected file does not exist.",
                 resources.getString("operation.import"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        FileMessage msg = new FileMessage(file, "unknown");
        if (!converter.importPossible(msg)) {
            JOptionPane.showMessageDialog
                (getFrame(),
                 "The converter is unable to import this document.",
                 resources.getString("operation.import"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        // select destination schema
        LinkedList schemas = new LinkedList();
        SchemaManager schemaManager = getApplication().getSchemaManager();
        for (Enumeration enum=schemaManager.getSchemaNames();
             enum.hasMoreElements();) {
            String name = (String)enum.nextElement();
            DocumentType schema = schemaManager.getSchema(name);
            if (converter.importPossible(msg, schema))
                schemas.add(schema);
        }
        DocumentType schema;
        if (schemas.isEmpty()) {
            // no schema found - maybe the converter doesn't want one
            schema = null;
        } else if (schemas.size()==1) {
            // only one schema fits, so take it automatically
            schema = (DocumentType)schemas.iterator().next();
        } else {
            // more than one schema fits, ask the user
            schema = SchemaChooser.chooseSchema(getFrame(), schemas);
            if (schema==null)
                return false;
        }
        // convert
        EmbeddedManager embeddedManager = new NewEmbeddedManager
            (resources.getString("importedDocumentName"));
        try {
            converter.importDocument
                (msg, schema, config, embeddedManager);
        } catch (ConverterException e) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(e),
                 resources.getString("operation.import"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        try {
            readFrom(embeddedManager);
        } catch (IOException e) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(e),
                 resources.getString("operation.import"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        } catch (SAXException e) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(e),
                 resources.getString("operation.import"), 
                 JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
        return true;
    }
    public boolean isNew(EditorDocument doc) {
        return doc.getEmbeddedManager() instanceof NewEmbeddedManager;
    }
    public boolean save(EditorDocument doc) {
        if (isNew(doc))
            return saveAs(doc);
        return super.save(doc);
    }
    public void openFile() {
        if (!documentClosing()) return;
        fileChooser.getUI().rescanCurrentDirectory(fileChooser);
        final int chooseRet = fileChooser.showOpenDialog(getFrame());
        if (chooseRet!=fileChooser.APPROVE_OPTION)
            return;
        File f = fileChooser.getSelectedFile();
        if (f.exists()) try {
            readFrom(f);
        } catch (IOException ee) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(ee), 
                 resources.getString("operation.open"), 
                 JOptionPane.ERROR_MESSAGE, null);
        } catch (SAXException e) {
            JOptionPane.showMessageDialog
                (getFrame(), 
                 resources.getString("error.file.xml.syntax")+'\n'+
                 ErrorUtil.format(e),
                 resources.getString("operation.open"), 
                 JOptionPane.ERROR_MESSAGE, null);
        }
        else {
            JOptionPane.showMessageDialog
                (getFrame(), 
                 resources.getString("error.file.exist")+'\n',
                 resources.getString("operation.open"), 
                 JOptionPane.ERROR_MESSAGE, null); 
        }
    }
    /*Actions***************************************/
    private DocumentAction dumpAction = new DumpAction();
    private DocumentAction saveAsAction = new SaveAsAction();
    private DocumentAction exportAction = new ExportAction();
    private DocumentAction viewsAction = new ViewsAction();
    private DocumentAction closeFileAction = new CloseFileAction();
    private Action[] defaultActions = {
        new NewChooseAction(),
        new OpenAction(),
        saveAsAction,
        exportAction,
        new ImportAction(),
        closeFileAction,
    };
    class NewChooseAction extends AbstractAction {
        NewChooseAction() {
            super("new");
        }
        public void actionPerformed(ActionEvent evt) { 
            if (!documentClosing())
                return;
            SchemaManager schemaManager = getApplication().getSchemaManager();
            Collection schemas = new LinkedList();
            for (Enumeration enum=schemaManager.getSchemaNames();
                 enum.hasMoreElements();) {
                String name = (String)enum.nextElement();
                DocumentType schema = schemaManager.getSchema(name);
                schemas.add(schema);
            }
            EditorDocumentType schema
                = (EditorDocumentType)SchemaChooser.chooseSchema
                (getFrame(), schemas);
            if (schema==null)
                return;
            Document doc = schema.createNewDocument(getSession());
            EmbeddedManager em = new NewEmbeddedManager
                (resources.getString("newDocumentName"));
            setDocument(new EditorDocument(doc.getDocumentElement(),
                                           schema, em));
            changed = false;
        }
    }
    class NewAction extends AbstractAction {
        String schema;
        NewAction(String _schema) { 
            super("new-"+_schema); 
            schema = _schema;
        }
        public void actionPerformed(ActionEvent evt) {
            if (!documentClosing()) return;
            try {
                // load document type
                EditorDocumentType edt = (EditorDocumentType)getApplication()
                    .getSchemaManager().getSchema(schema);
                Document doc;
                if (edt==null) {
                    // no doctype found? -> create empty document
                    doc = DOMUtil.createDocument();
                    doc.appendChild(doc.createElement(schema));
                } else {
                    // let doctype create template
                    doc = edt.createNewDocument(getSession());
                }
                // when creating new document, everything comes into memory
                // so we create a NewEmbeddedManager
                EmbeddedManager em = new NewEmbeddedManager
                    (resources.getString("newDocumentName"));
                setDocument(new EditorDocument(doc.getDocumentElement(),
                                               edt, em));
                // set some flags
                changed = false;
            } catch (Exception ex) {
                Debug.report(ex);
                JOptionPane.showMessageDialog
                    (getFrame(), ErrorUtil.format(ex), 
                     resources.getString("operation.new"),
                     JOptionPane.ERROR_MESSAGE, null);
            }
        }
    }
    class OpenAction extends AbstractAction {
        OpenAction() { super("open"); }
        public void actionPerformed(ActionEvent e) {
            openFile();
        }
    }
    class SaveAsAction extends DocumentAction {
        SaveAsAction() { super("saveas"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) { 
            saveAs(doc); 
        }
    }
    class ExportAction extends DocumentAction {
        ExportAction() { super("export"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) { 
            exportFile(doc); 
        }
    }
    class ImportAction extends AbstractAction {
        ImportAction() { super("import"); }
        public void actionPerformed(ActionEvent evt) { 
            importFile(); 
        }
    }
    class CloseFileAction extends DocumentAction {
        CloseFileAction() { super("close-file"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) {
            if (!documentClosing()) return;
            setDocument(null);
            changed = false;
        }
    }
}
