/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TopEditorFrame.java,v 1.30 2002/03/12 21:19:36 max Exp $
 *
 */

package org.leanedit;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.File;
import java.text.ParseException;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.leanedit.plugins.EditorPlugin;
import org.leanedit.plugins.ViewSelector;
import make.xml.SchemaManager;
import make.xml.ConfigPath;
import make.util.Debug;
import make.util.ErrorUtil;
import make.io.Streams;
import make.swing.ButtonManager;
import make.swing.ThreadAction;
import make.swing.bt.FrameDescription;
import make.swing.progress.ProgressIndicator;
import make.swing.panel.InterfacePanel;
import make.datamodel.embedded.EmbeddedManager;
import make.datamodel.embedded.DebugEmbeddedManager;
import make.datamodel.embedded.ProxyEmbeddedManager;
import make.datamodel.embedded.FileEmbeddedManager;
import make.datamodel.embedded.ZipEmbeddedManager;
import make.datamodel.embedded.MemoryEmbeddedManager;

/**
 * TopEditorFrame is a more interactive EditorFrame implementation
 * than EditorPanel. It provides buttons which let the user save
 * the document or switch to another view.
 */
public class TopEditorFrame extends EditorPanel {
    /*properties******************************/
    protected FrameDescription frameDescription;
    /*Constructor*****************************/
    public TopEditorFrame(EditorSession _sess, InterfacePanel _ip) {
        super(_sess, _ip);
        // default actions
        getInterfacePanel().getButtonManager().installActions(defaultActions);
        // frame description
        if (Debug.LEAK_DEBUG)
            System.err.println("+tef "+this);
    }
    protected void finalize() throws Throwable {
        super.finalize();
        if (Debug.LEAK_DEBUG)
            System.err.println("-tef "+this);
    }
    /*EditorPanel*********************************/
    protected void setEditorPlugin(EditorPlugin _plugin, EditorDocumentView _v) {
        if (frameDescription!=null) {
            getInterfacePanel().removeFrameDescription(frameDescription);
            frameDescription = null;
        }
        super.setEditorPlugin(_plugin, _v);
        if (editorComponent!=null) {
            // frame description
            Object fd = getView().getConfigPath().getDescription("frame");
            if (fd!=null) {
                if (fd instanceof Element) {
                    frameDescription = new FrameDescription
                        ((Element)fd, getClass(), getSession().getLocale());
                    getView().getConfigPath().putDescription("frame", frameDescription);
                } else
                    frameDescription = (FrameDescription)fd;
                getInterfacePanel().addFrameDescription(frameDescription);
            }
        }
    }
    public void setDocument(EditorDocument _doc, EditorPlugin _plugin, 
                            EditorDocumentView _v) {
        super.setDocument(_doc, _plugin, _v);
        dumpAction.update();
        saveAction.update();
        viewsAction.update();
        generateIdAction.update();
    }
    protected boolean documentClosing() {
        if (changed) {
            switch (JOptionPane.showConfirmDialog
                    (getFrame(), 
                     resources.getString("ask.changedSave"),
                     resources.getString("operation.close"), 
                     JOptionPane.YES_NO_CANCEL_OPTION,
                     JOptionPane.QUESTION_MESSAGE)) {
            case JOptionPane.CANCEL_OPTION: 
                return false;
            case JOptionPane.YES_OPTION: {
                EditorDocument doc = getDocument();
                if (doc==null || save(doc)) 
                    return false;
            }
            }
        }
        return true;
    }
    /*I/O*****************************************/
    public boolean saveAll(EditorDocument doc, EmbeddedManager em) 
        throws IOException {
        EmbeddedManager oem = doc.getEmbeddedManager();
        boolean ok = saveDocument(doc, em);
        doc.putNewEmbeddedManager(em);
        return ok;
    }
    public boolean saveDocument(EditorDocument doc) throws IOException {
        return saveDocument(doc, doc.getEmbeddedManager());
    }
    public boolean writeDocument(EditorDocument doc, EmbeddedManager em) 
        throws IOException {
        ProgressIndicator pi = getSession().getProgressIndicatorFactory()
            .create(resources.getString("message.save"),
                    resources.getString("operation.save"), 
                    true, null);
        try {
            doc.writeDocument(em);
        } finally {
            pi.close();
        }
        return true;
    }
    public boolean saveDocument(EditorDocument doc, EmbeddedManager em) 
        throws IOException {
        if (writeDocument(doc, em)) {
            changed = false;
            return true;
        } else
            return false;
    }
    public static EmbeddedManager createEmbeddedManager(File f, SchemaManager fm) 
        throws IOException {
        if (f.getName().endsWith(".zip"))
            return new ZipEmbeddedManager(f, fm, Streams.defaultEncoding);
        else
            return new FileEmbeddedManager(f, fm, Streams.defaultEncoding);
    }
    public boolean readFrom(File f) throws IOException, SAXException {
        return readFrom(createEmbeddedManager(f, getApplication().getSchemaManager()));
    }
    public boolean readFrom(EmbeddedManager _em) 
        throws IOException, SAXException {
        // parse
        Node n = _em.getDocument();
        if (n==null)
            throw new IOException(resources.getString("error.file.empty"));
        // load document type
        SchemaManager fm = getApplication().getSchemaManager();
        String tn = fm.getSchemaName(n.getOwnerDocument());
        EditorDocumentType docType = tn==null
            ?null
            :(EditorDocumentType)fm.getSchema(tn);
        // set document
        setDocument(new EditorDocument(n, docType, _em));
        changed = false;
        return true;
    }
    /*PanelContent**************************/
    public void panelClosed() {
        getInterfacePanel().getButtonManager().uninstallActions(defaultActions);
        if (frameDescription!=null) {
            getInterfacePanel().removeFrameDescription(frameDescription);
            frameDescription = null;
        }
        super.panelClosed();
    }
    /*File*****************************************/
    public boolean save(EditorDocument doc) {
        try {
            return saveDocument(doc);
        } catch (IOException ex) {
            JOptionPane.showMessageDialog
                (getFrame(), ErrorUtil.format(ex),
                 resources.getString("operation.save"), JOptionPane.ERROR_MESSAGE, null);
            return false;
        }
    }
    /*Actions***************************************/
    private DocumentAction dumpAction = new DumpAction();
    private DocumentAction saveAction = new SaveAction();
    private DocumentAction generateIdAction = new GenerateIdAction();
    private DocumentAction viewsAction = new ViewsAction();
    private Action[] defaultActions = {
        new ThreadAction(viewsAction),
        dumpAction,
        new DumpButtonManagerAction(),
        new ThreadAction(saveAction),
        new ThreadAction(generateIdAction),
    };
    class DumpAction extends DocumentAction {
        DumpAction() { super("dump"); }
        public void actionPerformed(ActionEvent evt, EditorDocument doc) { 
            try {
                EmbeddedManager em = new DebugEmbeddedManager
                    (getApplication().getSchemaManager());
                writeDocument(doc, em);
            } catch (IOException e) {
                Debug.report(e);
            }
        }
    }
    class DumpButtonManagerAction extends AbstractAction {
        DumpButtonManagerAction() { super("dump-button-manager"); }
        public void actionPerformed(ActionEvent evt) { 
            getInterfacePanel().getButtonManager().dump("");
        }
    }
    class ViewsAction extends DocumentAction {
        ViewsAction() { super("views"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) {
            if (doc.getType()==null)
                return;
            EditorDocumentView v = ViewSelector.selectView
                (getFrame(), 
                 resources.getString("operation.viewselect"),
                 resources.getString("prompt.viewselect"),
                 doc.getType().getViews());
            if (v!=null)
                setView(v);
        }
        public void update(EditorDocument doc) {
            setEnabled(doc!=null && doc.getType()!=null);
        }
    }
    class SaveAction extends DocumentAction {
        SaveAction() { super("save"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) { 
            save(doc); 
        }
    }
    class GenerateIdAction extends DocumentAction {
        GenerateIdAction() { super("generate-id"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) {
            if (JOptionPane.showConfirmDialog
                (getFrame(),
                 resources.getString("ask.generateId"),
                 resources.getString("operation.generateId"),
                 JOptionPane.YES_NO_OPTION, 
                 JOptionPane.QUESTION_MESSAGE)!=JOptionPane.YES_OPTION)
                return;
            try {
                editorComponent.flushDocument();
                flushRequired = false;
            } catch (ParseException e2) {
            }
            long newId = new java.util.Random().nextInt();
            ((Element)doc.getNode()).setAttribute("id", ""+newId);
            editorComponent.setDocument(doc);
            EmbeddedManager em = doc.getEmbeddedManager();
            while (em instanceof ProxyEmbeddedManager)
                em = ((ProxyEmbeddedManager)em).getOriginal();
            if (!(em instanceof FileEmbeddedManager ||
                  em instanceof ZipEmbeddedManager ||
                  em instanceof MemoryEmbeddedManager)) {
                em = new MemoryEmbeddedManager(resources.getString("newDocumentName"));
                doc.putNewEmbeddedManager(em);
                try {
                    writeDocument(doc, em);
                } catch (IOException ee) {
                    // must not happen when using MemoryEmbeddedManager
                    Debug.report(ee);
                }
                //isNew = true; FIXME
                getInterfacePanel().setTitle(em.toString());
                TopEditorFrame.this.firePropertyChange(DOCUMENT, doc, null);
                TopEditorFrame.this.firePropertyChange(DOCUMENT, null, doc);
            }
            changed = true;
        }
    }
}
