/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: RTFConverter.java,v 1.4 2002/03/12 21:18:48 max Exp $
 *
 */

package org.leanedit.convert;

import java.util.Hashtable;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.InputStream;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import make.io.Streams;
import make.mime.Message;
import make.mime.ArrayMessage;
import make.mime.MIMEUtil;
import make.xml.DocumentType;
import make.xml.ConfigPath;
import make.xml.RtfToXml;
import make.datamodel.embedded.EmbeddedManager;

/**
 * This converter can convert between XML and RTF (Rich Text Format).
 */
public class RTFConverter implements Converter {
    /*static***************************/
    protected static TransformerFactory tFactory;
    /*Converter******************************/
    public boolean importPossible() {
        return true;
    }
    public boolean exportPossible() {
        return true;
    }
    public boolean identify(String sysId) {
        return sysId.toLowerCase().endsWith(".rtf");
    }
    public boolean identify(Message msg) {
        return MIMEUtil.isMediaType(msg, "unknown") ||
            MIMEUtil.isContentType(msg, "application/rtf");
    }
    public boolean importPossible(Message msg) {
        return identify(msg);
    }
    public boolean importPossible(Message msg, DocumentType schema) {
        if (!importPossible(msg))
            return false;
        ConfigPath configPath = schema.getConfigPath();
        return configPath.getDescription("rtfimport")!=null;
    }
    public boolean exportPossible(DocumentType schema) {
        ConfigPath configPath = schema.getConfigPath();
        return configPath.open("export_rtf.xsl")!=null;
    }
    public void importDocument(Message src, DocumentType schema,
                               Hashtable config, EmbeddedManager dest)
        throws ConverterException {
        if (schema==null)
            throw new ConverterException("no schema specified");
        // parse the rtfimport configuration
        ConfigPath configPath = schema.getConfigPath();
        Element rtfImportConfig = (Element)configPath.getDescription("rtfimport");
        if (rtfImportConfig==null)
            throw new ConverterException("rtfimport.xml does not exist for this schema");
        // now convert
        RtfToXml rtf = new RtfToXml(rtfImportConfig);
        try {
            Reader reader = Streams.streamToReader(src.getInputStream());
            Document doc = rtf.translate(reader);
            dest.replaceDocument(doc.getDocumentElement());
        } catch (IOException e) {
            throw new ConverterException(e);
        }
    }
    public Message exportDocument(EmbeddedManager src, DocumentType schema,
                                  Hashtable config)
        throws ConverterException {
        Document doc;
        try {
            Node node = src.getDocument();
            if (node==null)
                throw new ConverterException("the document is empty");
            doc = node.getOwnerDocument();
        } catch (IOException e) {
            throw new ConverterException(e);
        } catch (SAXException e) {
            throw new ConverterException(e);
        }
        // find the rtf export style
        ConfigPath configPath = schema.getConfigPath();
        InputStream styleStream = configPath.open("export_rtf.xsl");
        if (styleStream==null)
            throw new ConverterException("export_rtf.xsl does not exist for this schema");
        // transform
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            DOMSource in = new DOMSource(doc);
            StreamSource style = new StreamSource(styleStream);
            StreamResult out = new StreamResult(baos);
            if (tFactory==null)
                tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer(style);
            transformer.transform(in, out);
        } catch (TransformerException e) {
            throw new ConverterException(e);
        }
        // return
        byte[] data = baos.toByteArray();
        return new ArrayMessage(data, "application/rtf");
    }
}
