/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2002 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorInputLinkFactory.java,v 1.11 2002/03/12 21:16:03 max Exp $
 *
 */

package org.leanedit.field;

import java.lang.reflect.Constructor;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Locale;
import java.io.FilenameFilter;
import org.w3c.dom.Element;
import make.xml.DOMUtil;
import make.util.Strings;
import make.util.Debug;
import make.datamodel.slot.Slot;
import make.datamodel.slot.SlotContainer;
import make.swing.field.Field;
import make.swing.il.InputLink;
import make.swing.il.InputLinkFactory;
import make.swing.il.DefaultInputLinkFactory;
import org.leanedit.EditorDocument;
import org.leanedit.EditorDocumentType;
import org.leanedit.EditorSession;
import org.leanedit.EditorFrame;
import org.leanedit.module.EditorModule;

public class EditorInputLinkFactory extends DefaultInputLinkFactory {
    /*static*********************************/
    private static Hashtable registeredModules = new Hashtable();
    public static void registerModule(EditorModule mod, Class cls) {
        try {
            Class[] paramClasses = { 
                EditorModule.class, EditorInputLinkFactory.class
            };
            Constructor constr = cls.getConstructor(paramClasses);
            registeredModules.put(mod, constr);
        } catch (Exception e) {
            Debug.report(e);
        }
    }
    public static void unregisterModule(EditorModule mod, Class cls) {
        registeredModules.remove(mod);
    }
    /*properties****************************/
    private EditorFrame frame;
    protected EditorDocument document;
    protected EditorDocumentType documentType;
    protected InputLinkFactory[] modules;
    /*Constructor**************************/
    public EditorInputLinkFactory(EditorFrame _frm, EditorDocumentType _type,
                                  SlotContainer _sc) {
        super(_sc);
        frame = _frm;
        documentType = _type;
        synchronized(registeredModules) {
            modules = new InputLinkFactory[registeredModules.size()];
            Enumeration enum = registeredModules.keys();
            for (int z=0; z<modules.length; z++) {
                EditorModule module = (EditorModule)enum.nextElement();
                Constructor constr = (Constructor)registeredModules.get(module);
                Object[] params = {
                    module, this
                };
                try {
                    modules[z] = (InputLinkFactory)constr.newInstance(params);
                } catch (Exception e) {
                    Debug.report(e, "while creating ilf for module "+module);
                    modules[z] = null;
                }
            }
        }
    }
    public EditorInputLinkFactory(EditorFrame _frm, EditorDocument _doc,
                                  SlotContainer _sc) {
        this(_frm, _doc.getType(), _sc);
        document = _doc; 
    }
    /*Info******************************/
    public EditorFrame getFrame() {
        return frame;
    }
    public EditorSession getSession() {
        return frame.getSession();
    }
    public EditorDocument getDocument() {
        return document;
    }
    public EditorDocumentType getDocumentType() {
        return documentType;
    }
    public Slot getSlot(Element vd) {
        return super.getSlot(vd);
    }
    /*Internal******************************/
    protected FilenameFilter loadFilenameFilter(Element vd) {
        Element[] es = DOMUtil.getChildElementsNS(vd, XML_NAMESPACE, "filetype");
        if (es.length==0)
            return null;
        String[] ext = new String[es.length];
        for (int z=0; z<es.length; z++)
            ext[z] = es[z].getAttributeNS(XML_NAMESPACE, "extension");
        return new ExtensionFilenameFilter(ext);
    }
    protected String getListName(Element vd) {
        String name;
        name = Strings.trim(vd.getAttributeNS(XML_NAMESPACE, "list"));
        if (name!=null)
            return name;
        Element list = DOMUtil.getUniqueTagNS(vd, XML_NAMESPACE, "list");
        if (list!=null) {
            name = Strings.trim(list.getAttributeNS(XML_NAMESPACE, "name"));
            if (name!=null)
                return name;
        }
        name = Strings.trim(vd.getAttributeNS(XML_NAMESPACE, "name"));
        if (name!=null)
            return name;
        return null;
    }
    /*DefaultInputLinkFactory******************/
    protected Object getList(Element vd, Locale locale) {
        if (documentType==null) {
            if (Debug.DEBUG)
                System.err.println("DEBUG: in EILF.getList: documentType==null");
            return null;
        }
        String name = getListName(vd);
        return name==null
            ?null
            :documentType.getList(name);
    }
    protected Field createField(Element vd, Locale locale) {
        String type = getType(vd);
        if (document!=null) {
            if (type.equals("embeddedselect"))
                return new EmbeddedSelectField(document.getEmbeddedManager(), 
                                               loadFilenameFilter(vd));
            if (type.equals("embedded"))
                return new EmbeddedField(document.getEmbeddedManager(), 
                                         loadFilenameFilter(vd));
        }
        if (type.equals("radio"))
            return new RadioField(vd);
        return super.createField(vd, locale);
    }
    public InputLink createInputLink(Element vd, Locale locale) {
        try {
            // super
            InputLink il = super.createInputLink(vd, locale);
            if (il!=null)
                return il;
            // some other type
            String type = getType(vd);
            if (type.equals("plugin"))
                return new PluginInputLink(vd, locale, document, frame);
            // ask module
            for (int z=0; z<modules.length; z++)
                if (modules[z]!=null) {
                    il = modules[z].createInputLink(vd, locale);
                    if (il!=null) return il;
                }
            if (Debug.DEBUG)
                System.err.println("WARNING: could not create il "+type);
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
        return null;
    }
}
