/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: DefaultEditorModule.java,v 1.8 2001/10/24 22:46:45 max Exp $
 *
 */

package org.leanedit.module;

import java.io.IOException;
import java.io.InputStream;
import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import make.util.Debug;
import make.xml.DOMUtil;
import make.xml.ConfigPath;
import make.xml.SchemaManager;
import make.module.ResourceLoader;
import make.module.PrefixResourceLoader;
import make.module.ModuleClassLoader;
import org.leanedit.EditorApplication;
import org.leanedit.EditorSession;
import org.leanedit.EditorFrame;
import org.leanedit.EditorDocumentType;
import org.leanedit.plugins.PluginLoader;
import org.leanedit.plugins.PluginManager;
import org.leanedit.convert.ConverterLoader;
import org.leanedit.convert.ConverterManager;

/**
 * This is a wrapper that loads the file leanedit.xml from
 * a ResourceLoader and loads some default configuration.
 * At the end, it tries to load the correct EditorModule
 * instance from the ResourceLoader - if there is one.
 */
public class DefaultEditorModule implements EditorModule {
    /*properties***********************/
    protected String name;
    protected EditorModule innerModule;
    protected EditorDocumentType[] schemas;
    protected PluginLoader[] plugins;
    protected ConverterLoader[] converters;
    /*Constructor*********************/
    public DefaultEditorModule(EditorApplication app, ResourceLoader rl)
	throws IOException, SAXException {
	// parse leanedit.xml
	InputStream is = openConfig(app, rl);
	if (is==null)
	    throw new IOException("module config file LEANEDIT-INF/leanedit.xml not found");
	Document doc = DOMUtil.parse(is);
	Element root = doc.getDocumentElement();
	// misc info
	name = DOMUtil.getElementString(root, "name");
	if (name==null) {
	    name = rl.toString();
	    System.err.println("WARNING: module "+name+" has no name");
	}
	// schemas
	Element[] registerSchemas = DOMUtil.getChildElements(root, "register-schema");
	if (registerSchemas.length>0)
	    schemas = new EditorDocumentType[registerSchemas.length];
	for (int z=0; z<registerSchemas.length; z++) {
	    String url = registerSchemas[z].getAttribute("url");
	    if (url==null) continue;
	    while (url.charAt(0)=='/') url = url.substring(1);
	    if (url.length()==0) continue;
	    ConfigPath configPath = openSchema(app, rl, url);
	    if (configPath==null) continue;
	    schemas[z] = new EditorDocumentType(app, configPath);
	}
	// create class loader
	ClassLoader classLoader = null;
	// plugins
	Element[] registerPlugins = DOMUtil.getChildElements(root, "register-plugin");
	if (registerPlugins.length>0)
	    plugins = new PluginLoader[registerPlugins.length];
	for (int z=0; z<registerPlugins.length; z++) {
	    String name = registerPlugins[z].getAttribute("name");
	    String displayName = DOMUtil.getElementString
		(registerPlugins[z], "display-name");
	    String description = DOMUtil.getElementString
		(registerPlugins[z], "description");
	    String className = DOMUtil.getElementString
		(registerPlugins[z], "class-ref");
	    if (classLoader==null)
		classLoader = openClasses(app, rl);
	    plugins[z] = new PluginLoader(name, displayName, description,
					  classLoader, className);
	}
	// converters
	Element[] registerConverters = DOMUtil.getChildElements(root, "register-converter");
	if (registerConverters.length>0)
	    converters = new ConverterLoader[registerConverters.length];
	for (int z=0; z<registerConverters.length; z++) {
	    String displayName = DOMUtil.getElementString
		(registerConverters[z], "display-name");
	    String description = DOMUtil.getElementString
		(registerConverters[z], "description");
	    String className = DOMUtil.getElementString
		(registerConverters[z], "class-ref");
	    if (classLoader==null)
		classLoader = openClasses(app, rl);
	    converters[z] = new ConverterLoader(displayName, description,
						classLoader, className);
	}
	// inner module
	String className = DOMUtil.getElementString(root, "module-class");
	if (className!=null) {
	    // load class + instantiate
	    try {
		if (classLoader==null)
		    classLoader = openClasses(app, rl);
		Class cls = classLoader.loadClass(className);
		innerModule = (EditorModule)cls.newInstance();
		name = null;
	    } catch (ClassNotFoundException e) {
		Debug.report(e, "while trying to load module "+className);
	    } catch (InstantiationException e) {
		Debug.report(e, "while trying to load module "+className);
	    } catch (IllegalAccessException e) {
		Debug.report(e, "while trying to load module "+className);
	    } catch (RuntimeException e) {
		Debug.report(e, "while trying to load module "+className);
	    } catch (Error e) {
		Debug.report(e, "while trying to load module "+className);
		throw e;
	    }
	}
	
    }
    /*DefaultEditorModule***************************/
    protected InputStream openConfig(EditorApplication app, 
				     ResourceLoader rl) {
	return rl.getResourceAsStream("/LEANEDIT-INF/leanedit.xml");
    }
    protected ConfigPath openSchema(EditorApplication app, 
				    ResourceLoader rl, String url) {
	return new ConfigPath
	    (rl, "/LEANEDIT-INF/schemas/"+url,
	     app.getEntityResolver());
    }
    protected ClassLoader openClasses(EditorApplication app, 
				      ResourceLoader rl) {
	PrefixResourceLoader prl
	    = new PrefixResourceLoader(rl, "LEANEDIT-INF/classes");
	return new ModuleClassLoader(app.getClass().getClassLoader(), prl);
    }
    /*EditorModule***************************/
    public String getName() {
	return innerModule==null
	    ? name
	    : innerModule.getName();
    }
    public void install(EditorApplication app) {
	if (plugins!=null) {
	    PluginManager pluginManager = app.getPluginManager();
	    for (int z=0; z<plugins.length; z++)
		pluginManager.registerPlugin(plugins[z]);
	}
	if (converters!=null) {
	    ConverterManager converterManager = app.getConverterManager();
	    for (int z=0; z<converters.length; z++)
		converterManager.registerConverter(converters[z]);
	}
	if (schemas!=null) {
	    SchemaManager schemaManager = app.getSchemaManager();
	    for (int z=0; z<schemas.length; z++) {
		if (schemas[z]!=null)
		    schemaManager.registerSchema(schemas[z]);
	    }
	}
	if (innerModule!=null)
	    innerModule.install(app);
    }
    public void uninstall(EditorApplication app) {
	if (innerModule!=null)
	    innerModule.uninstall(app);
	if (schemas!=null) {
	    SchemaManager schemaManager = app.getSchemaManager();
	    for (int z=0; z<schemas.length; z++) {
		if (schemas[z]!=null)
		    schemaManager.unregisterSchema(schemas[z]);
	    }
	}
	if (converters!=null) {
	    ConverterManager converterManager = app.getConverterManager();
	    for (int z=0; z<converters.length; z++)
		converterManager.unregisterConverter(converters[z]);
	}
	if (plugins!=null) {
	    PluginManager pluginManager = app.getPluginManager();
	    for (int z=0; z<plugins.length; z++)
		pluginManager.unregisterPlugin(plugins[z]);
	}
    }
    public void install(EditorSession sess) {
	if (innerModule!=null)
	    innerModule.install(sess);
    }
    public void uninstall(EditorSession sess) {
	if (innerModule!=null)
	    innerModule.uninstall(sess);
    }
    public void install(EditorFrame frm) {
	if (innerModule!=null)
	    innerModule.install(frm);
    }
    public void uninstall(EditorFrame frm) {
	if (innerModule!=null)
	    innerModule.uninstall(frm);
    }
    public boolean open(EditorSession sess, Object key) {
	if (innerModule!=null)
	    return innerModule.open(sess, key);
	return false;
    }
}
