/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: DefaultEditorComponent.java,v 1.10 2001/12/22 11:50:38 max Exp $
 *
 */

package org.leanedit.plugins;

import java.awt.Component;
import java.awt.Frame;
import java.awt.Container;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.util.Vector;
import java.util.Enumeration;
import java.text.ParseException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.KeyStroke;
import javax.swing.JComponent;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.transform.TransformerException;
import org.apache.xpath.objects.XObject;
import make.xml.DOMUtil;
import make.swing.ButtonManager;
import make.swing.GenericButton;
import make.swing.KeyStrokeButton;
import make.swing.ButtonManager;
import make.swing.bt.ButtonContainer;
import make.swing.panel.InterfacePanel;
import make.util.Debug;
import make.util.Strings;
import org.leanedit.EditorDocument;
import org.leanedit.EditorDocumentView;
import org.leanedit.EditorFrame;
import org.leanedit.EditorSession;
import org.leanedit.EditorApplication;

public abstract class DefaultEditorComponent 
    implements EditorComponent, FocusListener {
    /*properties*********************************/
    protected EditorFrame editorFrame;
    protected EditorDocument document;
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private Vector listeners = new Vector();
    private Action[] installedActions;
    private Vector configActions;
    private Vector keyBindings;
    private boolean hasFocus = false;
    private boolean disposed = false;
    private Vector inputComponents = new Vector();
    /*Constructor******************************/
    public DefaultEditorComponent(EditorFrame _ef) {
        editorFrame = _ef;
    }
    /*Info*************************************/
    public EditorFrame getEditorFrame() { return editorFrame; }
    public InterfacePanel getInterfacePanel() { 
        return editorFrame.getInterfacePanel();
    }
    public ButtonContainer getButtonContainer() {
        return getInterfacePanel().getButtonContainer();
    }
    public ButtonManager getButtonManager() {
        return getInterfacePanel().getButtonManager();
    }
    public EditorDocumentView getView() { return editorFrame.getView(); }
    public EditorSession getSession() { return editorFrame.getSession(); }
    public EditorApplication getApplication() { return getSession().getApplication(); }
    public EditorDocument getDocument() { return document; }
    public Document getXMLDocument() {
        return getDocument().getNode().getOwnerDocument();
    }
    /*Info********************************/
    public Frame getFrame() {
        return editorFrame.getInterfacePanel().getFrame();
    }
    /*DefaultEditorComponent********************/
    protected void firePropertyChange(String name, Object o, Object n) {
        pcs.firePropertyChange(name, o, n);
    }
    protected void registerInputComponent(Component c) {
        c.addFocusListener(this);
        inputComponents.addElement(c);
        if (c instanceof Container) {
            Container con = (Container)c;
            Component[] ch = con.getComponents();
            for (int z=0; z<ch.length; z++)
                registerInputComponent(ch[z]);
        }
    }
    protected void unregisterInputComponent(Component c) {
        if (c instanceof Container) {
            Container con = (Container)c;
            Component[] ch = con.getComponents();
            for (int z=0; z<ch.length; z++)
                unregisterInputComponent(ch[z]);
        }
        inputComponents.removeElement(c);
        c.removeFocusListener(this);
    }
    protected void fireFocusGained() {
        ComponentEvent evt = new ComponentEvent(this);
        for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
            ((ComponentListener)enum.nextElement()).focusGained(evt);
    }
    protected void fireDocumentChanged(ComponentEvent evt) {
        for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
            ((ComponentListener)enum.nextElement()).documentChanged(evt);
    }
    protected void fireDocumentChanged(Node n) {
        fireDocumentChanged(new ComponentEvent(this, getDocument(), n));
    }
    protected void fireDocumentChanged() {
        fireDocumentChanged(getDocument().getNode());
    }
    /*Actions****************************/
    public Action[] getActions() { return null; }
    protected void installAction(Action _a) {
        if (configActions==null)
            configActions = new Vector();
        configActions.addElement(_a);
        if (hasFocus)
            editorFrame.getInterfacePanel().getButtonManager().installAction(_a);
    }
    protected void uninstallAction(Action _a) {
        configActions.removeElement(_a);
        if (hasFocus)
            editorFrame.getInterfacePanel().getButtonManager().uninstallAction(_a);
    }
    protected Object getViewDescription(String pluginName) {
        return getView().getConfigPath().getDescription
            (pluginName+'_'+document.getName());
    }
    protected void putViewDescription(String pluginName, Object vd) {
        getView().getConfigPath().putDescription
            (pluginName+'_'+document.getName(), vd);
    }
    protected Action loadAction(Element _a) {
        String tag = _a.getTagName();
        System.err.println("WARNING: unknown action "+tag);
        return null;
    }
    protected void loadActions(Element _a) {
        if (configActions!=null)
            while (!configActions.isEmpty())
                uninstallAction((Action)configActions.lastElement());
        if (_a!=null) {
            Element[] as = DOMUtil.getChildElements(_a);
            for (int z=0; z<as.length; z++) {
                Action a = loadAction(as[z]);
                if (a!=null)
                    installAction(a);
            }
        }
    }
    /*KeyBindings******************************/
    protected void installButton(GenericButton _gb) {
        if (keyBindings==null)
            keyBindings = new Vector();
        keyBindings.addElement(_gb);
        if (hasFocus)
            editorFrame.getInterfacePanel().getButtonManager().installButton(_gb);
    }
    protected void uninstallButton(GenericButton _gb) {
        keyBindings.removeElement(_gb);
        if (hasFocus)
            editorFrame.getInterfacePanel().getButtonManager().uninstallButton(_gb);
    }
    protected GenericButton loadKeyBinding(JComponent c, Element _ks) {
        return new KeyStrokeButton
            (c, _ks.getAttribute("action"),
             KeyStroke.getKeyStroke(_ks.getAttribute("key")),
             JComponent.WHEN_FOCUSED);
    }
    protected void loadKeyBindings(JComponent c, Element _a) {
        if (keyBindings!=null)
            for (Enumeration enum=keyBindings.elements(); enum.hasMoreElements();)
                uninstallButton((GenericButton)enum.nextElement());
        if (_a!=null) {
            Element[] as = DOMUtil.getChildElements(_a);
            for (int z=0; z<as.length; z++) {
                GenericButton a = loadKeyBinding(c, as[z]);
                if (a!=null)
                    installButton(a);
            }
        }
    }
    /*Find**************************************/
    protected XObject execute(String s) 
        throws TransformerException {
        return document.execute(s);
    }
    /*FocusListener****************************/
    public void focusGained(FocusEvent evt) {
        if (!evt.isTemporary() && !hasFocus)
            fireFocusGained();
    }
    public void focusLost(FocusEvent evt) {}
    /*Widget*******************************/
    public void setEnabled(boolean _e) {}
    public boolean getEnabled() { return true; }
    public abstract Component getComponent();
    public void dispose() {
        if (Debug.DEBUG) {
            if (disposed)
                throw new InternalError();
            disposed = true;
        }
        for (Enumeration enum=inputComponents.elements(); enum.hasMoreElements();)
            ((Component)enum.nextElement()).removeFocusListener(this);
        inputComponents = null;
        if (hasFocus)
            focusLost();
        editorFrame = null;
        document = null;
        listeners = null;
    }
    /*EditorComponent***************************/
    public void setDocument(EditorDocument doc) {
        EditorDocument oldDoc = document;
        document = doc;
        firePropertyChange(DOCUMENT, oldDoc, document);
    }
    public void flushDocument() throws ParseException {}
    public void focusGained() {
        if (Debug.DEBUG && hasFocus)
            throw new InternalError("focus error for "+this);
        if (Debug.DEBUG && installedActions!=null)
            throw new InternalError();
        installedActions = getActions();
        ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
        if (installedActions!=null)
            bm.installActions(installedActions);
        if (configActions!=null)
            bm.installActions(configActions.elements());
        if (keyBindings!=null)
            bm.installButtons(keyBindings.elements());
        hasFocus = true;
    }
    public void focusLost() {
        if (Debug.DEBUG && !hasFocus)
            throw new InternalError("focus error for "+this);
        ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
        if (keyBindings!=null)
            bm.uninstallButtons(keyBindings.elements());
        if (configActions!=null)
            bm.uninstallActions(configActions.elements());
        if (installedActions!=null) {
            bm.uninstallActions(installedActions);
            installedActions = null;
        }
        hasFocus = false;
    }
    public void addPropertyChangeListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener(l);
    }
    public void removePropertyChangeListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener(l);
    }
    public void addComponentListener(ComponentListener l) {
        listeners.addElement(l);
    }
    public void removeComponentListener(ComponentListener l) {
        listeners.addElement(l);
    }
    /*Actions********************************/
    protected abstract class ConfigAction extends AbstractAction {
        protected ConfigAction(String ac, Element vd) {
            super(ac);
            ac = Strings.trim(vd.getAttribute("command"));
            if (ac!=null)
                putValue(NAME, ac);
        }
    }
}
