/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ChecklistComponent.java,v 1.6 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.checklist;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Map;
import java.util.Iterator;
import java.awt.Component;
import java.awt.Dimension;
import java.text.ParseException;
import javax.swing.JComponent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import org.w3c.dom.Element;
import javax.xml.transform.TransformerException;
import make.util.Debug;
import make.xml.DOMUtil;
import make.datamodel.slot.Slot;
import org.leanedit.plugins.DefaultEditorComponent;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;
import make.swing.ButtonManager;
import make.swing.field.Field;
import make.swing.field.CheckBoxField;
import make.swing.field.FieldListener;
import make.swing.field.FieldEvent;
import make.swing.il.InputLink;
import make.swing.il.ContainerInputLink;
import make.swing.il.FieldInputLink;

public class ChecklistComponent extends DefaultEditorComponent {
    class ChangedListener implements FieldListener {
        /*properties************/
        protected Vector installedFields = new Vector();
        /*FieldListener***************/
        public void valueChanged(FieldEvent evt) {
            ChecklistComponent.this.fireDocumentChanged();
        }
        public void focusGained(FieldEvent evt) {}
        public void focusLost(FieldEvent evt) {}
        public void actionPerformed(FieldEvent evt) {}
        /*Public**********************/
        public void install(InputLink il) {
            if (il instanceof FieldInputLink) {
                FieldInputLink fil = (FieldInputLink)il;
                Field f = fil.getField();
                Slot s = fil.getSlot();
                f.addFieldListener(this);
                installedFields.addElement(f);
            } else if (il instanceof ContainerInputLink) {
                ContainerInputLink cil = (ContainerInputLink)il;
                for (Enumeration enum=cil.getInputLinks(); 
                     enum.hasMoreElements();) {
                    InputLink il2 = (InputLink)enum.nextElement();
                    install(il2);
                }
            }
        }
        public void uninstall() {
            for (Enumeration enum=installedFields.elements(); 
                 enum.hasMoreElements();) {
                Field f = (Field)enum.nextElement();
                f.removeFieldListener(this);
            }
            installedFields.removeAllElements();
        }
    }
    /*properties*****************************/
    // visual
    protected Box box;
    protected Vector fields = new Vector();
    // data
    protected ChangedListener changedListener = new ChangedListener();
    /*constructor***************************/
    public ChecklistComponent(EditorFrame _ef) {
        super(_ef);
        box = new Box(BoxLayout.Y_AXIS) {
                public Dimension getMaximumSize() {
                    return new Dimension(super.getMaximumSize().width,
                                         getPreferredSize().height);
                }
            };
    }
    /*Misc************************************/
    protected void addInputLink(InputLink il) {
        fields.addElement(il);
        Component c = il.getComponent();
        if (c instanceof JComponent) {
            JComponent jc = (JComponent)c;
            Dimension pd = jc.getPreferredSize();
            Dimension md = jc.getMaximumSize();
            Dimension d = new Dimension(md.width, pd.height);
            jc.setMaximumSize(d);
        }
        box.add(c);
        changedListener.install(il);
        il.install(editorFrame.getInterfacePanel().getButtonManager());
    }
    /*EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
        super.setDocument(_doc);
        final ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
        // remove old rows
        changedListener.uninstall();
        for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
            InputLink il = (InputLink)enum.nextElement();
            il.uninstall(bm);
            il.dispose();
        }
        fields.removeAllElements();
        unregisterInputComponent(box);
        Component[] oldRows = box.getComponents();
        for (int z=0; z<oldRows.length; z++)
            box.remove(oldRows[z]);
        // load new rows
        if (_doc!=null) {
            Element vd = (Element)getViewDescription("checklist");
            // load items
            Hashtable items = new Hashtable();
            Element[] itel = DOMUtil.getChildElements(vd, "item");
            for (int z=0; z<itel.length; z++) {
                String value = itel[z].getAttribute("value");
                String tag = DOMUtil.getElementString(itel[z], "tag");
                String namePath = DOMUtil.getElementString(itel[z], "name-path");
                Hashtable assign = new Hashtable();
                Element[] assignEls = DOMUtil.getChildElements(itel[z], "assign");
                for (int x=0; x<assignEls.length; x++) {
                    assign.put(assignEls[x].getAttribute("path"),
                               DOMUtil.getElementString(assignEls[x]));
                }
                try {
                    items.put(value, new ElementDescription(tag, namePath, assign));
                } catch (TransformerException e) {
                    Debug.report(e);
                }
            }
            ElementDescription trueDesc = (ElementDescription)items.get("yes");
            ElementDescription falseDesc = (ElementDescription)items.get("no");
            // load list
            Element list = DOMUtil.getUniqueTag(vd, "list");
            String listName = list.getAttribute("name");
            Map map = (Map)getDocument().getType().getList(listName);
            // sort out data
            Element container = (Element)_doc.getNode();
            Hashtable data = new Hashtable();
            Element[] dataEls = DOMUtil.getChildElements(container, trueDesc.getTag());
            for (int z=0; z<dataEls.length; z++) {
                String name = trueDesc.getName(dataEls[z]);
                if (name==null)
                    continue;
                data.put(name, dataEls[z]);
            }
            // and create fields
            Iterator it = map.entrySet().iterator();
            while (it.hasNext()) {
                Map.Entry entry = (Map.Entry)it.next();
                String name = (String)entry.getKey();
                String value = entry.getValue().toString();
                Field field = new CheckBoxField
                    (value, Boolean.TRUE, Boolean.FALSE);
                Slot slot = new ChecklistSlot(container, name,
                                              trueDesc, falseDesc,
                                              (Element)data.get(name));
                InputLink il = new FieldInputLink(null, slot, field, getSession().getLocale());
                addInputLink(il);
            }
        }
        registerInputComponent(box);
        // validate
        if (box.getParent()!=null)
            box.getParent().validate();
    }
    public void flushDocument() throws ParseException {
        for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
            InputLink field = (InputLink)enum.nextElement();
            field.apply();
        }
    }
    public Component getComponent() { return box; }
    public void dispose() {
        changedListener.uninstall();
        final ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
        for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
            InputLink il = (InputLink)enum.nextElement();
            il.uninstall(bm);
            il.dispose();
        }
        fields.removeAllElements();
        super.dispose();
    }
    /*DefaultEditorComponent********************/
    protected void fireDocumentChanged() {
        super.fireDocumentChanged();
    }
}
