/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: AbstractListComponent.java,v 1.8 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.container;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import javax.swing.Action;
import javax.swing.ListSelectionModel;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import make.swing.bt.FrameDescription;
import make.swing.panel.InterfacePanel;
import make.swing.panel.DialogInterfacePanel;
import org.leanedit.EditorFrame;
import org.leanedit.SubEditorFrame;
import org.leanedit.EditorDocument;
import org.leanedit.EditorDocumentView;
import org.leanedit.plugins.EditorPlugin;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.ContainerComponent;
import org.leanedit.plugins.ComponentListener;
import org.leanedit.plugins.ComponentEvent;
import org.leanedit.plugins.EditorCursorComponent;
import org.leanedit.plugins.EditorCursor;

public abstract class AbstractListComponent
    extends ContainerComponent 
    implements EditorCursorComponent {
    /*properties*****************************/
    protected DynamicContainerModel model;
    protected ListSelectionModel selectionModel;
    protected ContainerCursor rowSelectionCursor;
    /*Constructor****************************/
    public AbstractListComponent(EditorFrame _ef) {
        super(_ef);
    }
    public AbstractListComponent(EditorFrame _ef,
                                 DynamicContainerModel _m,
                                 ListSelectionModel _sm) {
        this(_ef);
        setModel(_m, _sm);
    }
    protected void setModel(DynamicContainerModel _m,
                            ListSelectionModel _sm) {
        model = _m; 
        selectionModel = _sm;
    }
    protected synchronized void itemChanged(EditorComponent ec, EditorDocument ed) {
        try {
            ec.flushDocument();
            model.itemChanged(model.indexOfItem(ed));
        } catch (ParseException e) {
        }
    }
    /*EditorComponent**********************************/
    public synchronized void setDocument(EditorDocument _doc) {
        super.setDocument(_doc);
        editElementAction = null;
    }
    /*EditorCursorComponent****************************/
    public EditorCursor getCursor(Object key) {
        if (rowSelectionCursor==null)
            rowSelectionCursor = new ContainerCursor
                (model, selectionModel);
        return rowSelectionCursor;
    }
    /*DefaultEditorComponent*******************/
    protected Action loadAction(Element _a) {
        final String tag = _a.getTagName();
        if (tag.equals("edit-element")) {
            editElementAction = new EditElementAction(_a);
            return editElementAction;
        }
        if (tag.equals("delete-element"))
            return new DeleteElementAction(_a);
        if (tag.equals("new-element"))
            return new InsertElementAction(_a.getAttribute("tag"), _a, true);
        if (tag.equals("move-up"))
            return new MoveRowUpAction(_a);
        if (tag.equals("move-down"))
            return new MoveRowDownAction(_a);
        return super.loadAction(_a);
    }
    protected void fireDocumentChanged(ComponentEvent evt) {
        super.fireDocumentChanged(evt);
    }
    /*AbstractListComponent*****************/
    protected void editElement(final EditorDocument ed) {
        if (ed==null || editElementAction==null)
            return;
        EditorPlugin ep = getApplication().getPluginManager()
            .loadPlugin(editElementAction.plugin);
        if (ep==null) return;
        DialogInterfacePanel ip = new DialogInterfacePanel(getFrame());
        FrameDescription fd;
        final String fdName = "frame_"+ed.getNode().getNodeName();
        EditorDocumentView view = getView();
        Object _fd = view.getConfigPath().getDescription(fdName);
        if (_fd instanceof FrameDescription) {
            fd = (FrameDescription)_fd;
        } else {
            fd = new FrameDescription((Element)_fd, getClass(),
                                      getSession().getLocale());
            view.getConfigPath().putDescription(fdName, fd);
        }
        SubEditorFrame ef = new SubEditorFrame(getEditorFrame(), ip, fd);
        ef.setDocument(ed, ep, view);
        final EditorComponent ec = ef.getComponents(EditorFrame.COMPONENT_ROOT)[0];
        ec.addComponentListener(new ComponentListener() {
                public void focusGained(ComponentEvent evt2) {}
                public void documentChanged(ComponentEvent evt2) {
                    itemChanged(ec, ed);
                    AbstractListComponent.this.fireDocumentChanged(evt2);
                }
            });
        ip.show();
    }
    protected void editElement(int idx) {
        if (idx==-1) return;
        editElement(model.getItem(idx));
    }
    /*Sel****************************/
    public int getSelectedIndex() {
        final int min = selectionModel.getMinSelectionIndex();
        if (min==-1)
            return -1;
        final int max = selectionModel.getMaxSelectionIndex();
        for (int z=min; z<=max; z++)
            if (selectionModel.isSelectedIndex(z))
                return z;
        return -1;
    }
    /*Actions********************************/
    private EditElementAction editElementAction;
    protected class DeleteElementAction extends ConfigAction {
        public DeleteElementAction(Element vd) { 
            super("delete-element", vd); 
        }
        public void actionPerformed(ActionEvent evt) {
            final int min = selectionModel.getMinSelectionIndex();
            if (min==-1)
                return;
            final int max = selectionModel.getMaxSelectionIndex();
            int oldSelection = getSelectedIndex();
            for (int z=max; z>=min; z--)
                if (selectionModel.isSelectedIndex(z))
                    model.removeItem(z);
            final int count = model.getItemCount();
            if (count>0) {
                if (oldSelection>=count)
                    oldSelection = count-1;
                if (oldSelection>=0)
                    selectionModel.setSelectionInterval(oldSelection, oldSelection);
            }
        }
    }
    protected class EditElementAction extends ConfigAction {
        /*properties******/
        protected String plugin;
        /*Constructor**********/
        public EditElementAction(Element vd) {
            super("edit-element", vd);
            plugin = vd.getAttribute("plugin");
        }
        /*ActionListener*******/
        public void actionPerformed(ActionEvent evt) {
            final int row = getSelectedIndex();
            if (row==-1) return;
            editElement(row);
        }
    }
    class MoveRowUpAction extends ConfigAction {
        public MoveRowUpAction(Element vd) { 
            super("move-up", vd); 
        }
        public void actionPerformed(ActionEvent evt) {
            final int row = getSelectedIndex();
            if (row==-1)
                return;
            final int targetRow = row-1;
            if (targetRow<0)
                return;
            final int newRow = model.moveItem(row, targetRow);
            if (newRow!=-1)
                selectionModel.setSelectionInterval(newRow, newRow);
        }
    }
    abstract class InsertNodeAction extends ConfigAction {
        protected boolean edit=false;
        public InsertNodeAction(String _ac, Element vd, boolean _edit) { 
            super(_ac, vd); 
            edit=_edit;
        }
        protected abstract Node createNode();
        public void actionPerformed(ActionEvent evt) {
            int row = getSelectedIndex();
            if (row==-1)
                row = model.getItemCount();
            Node n = createNode();
            if (n==null) return;
            EditorDocument doc = new EditorDocument(n, getDocument());
            row = model.insertItem(doc, row);
            if (row==-1) return;
            selectionModel.setSelectionInterval(row, row);
            if (edit)
                editElement(row);
        }
    }
    class InsertElementAction extends InsertNodeAction {
        String tag;
        public InsertElementAction(String _tag, Element vd, boolean _edit) { 
            super("new-"+_tag, vd, _edit); 
            tag = _tag;
        }
        protected Node createNode() {
            return getXMLDocument().createElement(tag);
        }
    }
    class MoveRowDownAction extends ConfigAction {
        public MoveRowDownAction(Element vd) { 
            super("move-down", vd); 
        }
        public void actionPerformed(ActionEvent evt) {
            final int row = getSelectedIndex();
            if (row==-1)
                return;
            final int targetRow = row+1;
            if (targetRow>model.getItemCount())
                return;
            final int newRow = model.moveItem(row, targetRow);
            if (newRow!=-1)
                selectionModel.setSelectionInterval(newRow, newRow);
        }
    }
}
