/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SimpleFormComponent.java,v 1.11 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.simpleform;

import java.util.Vector;
import java.util.Enumeration;
import java.awt.Component;
import java.awt.Dimension;
import java.text.ParseException;
import javax.swing.JComponent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import org.w3c.dom.Element;
import make.util.Debug;
import make.datamodel.slot.Slot;
import make.datamodel.slot.SlotContainer;
import make.xml.DOMUtil;
import make.xml.slot.DOMSlotContainer;
import make.xml.slot.DOMSlot;
import org.leanedit.plugins.DefaultEditorComponent;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;
import org.leanedit.field.EditorInputLinkFactory;
import make.swing.ButtonManager;
import make.swing.field.Field;
import make.swing.field.FieldListener;
import make.swing.field.FieldEvent;
import make.swing.il.InputLink;
import make.swing.il.InputLinkFactory;
import make.swing.il.ContainerInputLink;
import make.swing.il.FieldInputLink;

public class SimpleFormComponent extends DefaultEditorComponent {
    class ChangedListener implements FieldListener {
        /*properties************/
        protected Vector installedFields = new Vector();
        /*FieldListener***************/
        public void valueChanged(FieldEvent evt) {
            SimpleFormComponent.this.fireDocumentChanged();
        }
        public void focusGained(FieldEvent evt) {}
        public void focusLost(FieldEvent evt) {}
        public void actionPerformed(FieldEvent evt) {}
        /*Public**********************/
        public void install(InputLink il) {
            if (il instanceof FieldInputLink) {
                FieldInputLink fil = (FieldInputLink)il;
                Field f = fil.getField();
                Slot s = fil.getSlot();
                f.addFieldListener(this);
                installedFields.addElement(f);
            } else if (il instanceof ContainerInputLink) {
                ContainerInputLink cil = (ContainerInputLink)il;
                for (Enumeration enum=cil.getInputLinks(); 
                     enum.hasMoreElements();) {
                    InputLink il2 = (InputLink)enum.nextElement();
                    install(il2);
                }
            }
        }
        public void uninstall() {
            for (Enumeration enum=installedFields.elements(); 
                 enum.hasMoreElements();) {
                Field f = (Field)enum.nextElement();
                f.removeFieldListener(this);
            }
            installedFields.removeAllElements();
        }
    }
    /*properties*****************************/
    // visual
    protected Box box;
    protected Vector fields = new Vector();
    // data
    protected ChangedListener changedListener = new ChangedListener();
    /*constructor***************************/
    public SimpleFormComponent(EditorFrame _ef) {
        super(_ef);
        box = new Box(BoxLayout.Y_AXIS) {
                public Dimension getMaximumSize() {
                    return new Dimension(super.getMaximumSize().width,
                                         getPreferredSize().height);
                }
            };
        if (Debug.LEAK_DEBUG)
            System.err.println("+sec "+this);
    }
    protected void finalize() throws Throwable {
        super.finalize();
        if (Debug.LEAK_DEBUG)
            System.err.println("-sec "+this);
    }
    /*Info************************************/
    protected Field findField(InputLink il, String path) {
        if (il instanceof FieldInputLink) {
            FieldInputLink fil = (FieldInputLink)il;
            Slot s = fil.getSlot();
            if (s instanceof DOMSlot) {
                DOMSlot ds = (DOMSlot)s;
                if (path.equals(ds.getPath()))
                    return fil.getField();
            }
        } else if (il instanceof ContainerInputLink) {
            ContainerInputLink cil = (ContainerInputLink)il;
            for (Enumeration enum=cil.getInputLinks(); 
                 enum.hasMoreElements();) {
                InputLink il2 = (InputLink)enum.nextElement();
                Field f = findField(il2, path);
                if (f!=null)
                    return f;
            }
        }
        return null;
    }
    public Field getField(String path) {
        for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
            InputLink il = (InputLink)enum.nextElement();
            Field f = findField(il, path);
            if (f!=null)
                return f;
        }
        return null;
    }
    /*EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
        super.setDocument(_doc);
        final ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
        // remove old rows
        changedListener.uninstall();
        for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
            InputLink il = (InputLink)enum.nextElement();
            il.uninstall(bm);
            il.dispose();
        }
        fields.removeAllElements();
        unregisterInputComponent(box);
        Component[] oldRows = box.getComponents();
        for (int z=0; z<oldRows.length; z++)
            box.remove(oldRows[z]);
        // load new rows
        if (_doc!=null) {
            Element vd = (Element)getViewDescription("simpleform");
            Element f = DOMUtil.getUniqueTag(vd, "rows");
            if (f!=null) {
                SlotContainer sc
                    = new DOMSlotContainer(getDocument().getType(),
                                           getDocument().getNode());
                InputLinkFactory factory 
                    = new EditorInputLinkFactory(editorFrame, getDocument(), sc);
                Element[] fs = DOMUtil.getChildElements(f);
                for (int z=0; z<fs.length; z++) {
                    InputLink il = factory.createInputLink(fs[z], getSession().getLocale());
                    if (il!=null) {
                        fields.addElement(il);
                        Component c = il.getComponent();
                        if (c instanceof JComponent) {
                            JComponent jc = (JComponent)c;
                            Dimension pd = jc.getPreferredSize();
                            Dimension md = jc.getMaximumSize();
                            Dimension d = new Dimension(md.width, pd.height);
                            jc.setMaximumSize(d);
                        }
                        box.add(c);
                        changedListener.install(il);
                        il.install(bm);
                    }
                }
            }
        }
        registerInputComponent(box);
        // validate
        if (box.getParent()!=null)
            box.getParent().validate();
    }
    public void flushDocument() throws ParseException {
        for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
            InputLink field = (InputLink)enum.nextElement();
            field.apply();
        }
    }
    public Component getComponent() { return box; }
    public void dispose() {
        changedListener.uninstall();
        final ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
        for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
            InputLink il = (InputLink)enum.nextElement();
            il.uninstall(bm);
            il.dispose();
        }
        fields.removeAllElements();
        super.dispose();
    }
    /*DefaultEditorComponent********************/
    protected void fireDocumentChanged() {
        super.fireDocumentChanged();
    }
}
