/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SwingDocument.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.text.tree;

import java.util.Hashtable;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import org.leanedit.EditorDocument;
import org.leanedit.plugins.text.TextEditorStyleSet;
import make.datamodel.embedded.EmbeddedManager;
import make.util.Debug;
import make.xml.DOMUtil;
import make.xml.DocumentType;
import make.swing.text.Element;
import make.swing.text.AbstractDocument;
import make.swing.text.DocumentEvent;

public class SwingDocument extends AbstractDocument {
    /***properties**************************/
    // data
    protected SwingElementFactory elementFactory;
    protected EditorDocument editorDocument;
    // visual
    protected TextEditorStyleSet styles;
    /***Constructor****************************/
    public SwingDocument(EditorDocument _doc, TextEditorStyleSet _st) {
	editorDocument = _doc; styles = _st;
	elementFactory = new XmlElementFactory(this, editorDocument.getType(), _st);
	setRoot(new SwingBody((org.w3c.dom.Element)editorDocument.getNode(), this));
    }
    /***AbstractDocument**************************/
    protected Element createStyleElement(String style) {
	return new StyleElement(this, style);
    }
    /***Document********************************/
    private Element getParagraphElement(Element el, int offs, Position.Bias bias)
	throws BadLocationException {
	if (!el.isLeaf()) {
	    Element ch = el.getElementAt(offs, bias);
	    if (ch!=null) {
		ch = getParagraphElement(ch, offs, bias);
		if (ch!=null) return ch;
	    }
	}
	if (el instanceof Paragraph)
	    return el;
	return null;
    }
    public Element getParagraphElement(int offs, Position.Bias bias) 
	throws BadLocationException {
	return getParagraphElement(getRootElement(), offs, bias);
    }
    public Hashtable getStyle(String name) { 
	return styles.getStyle(name); 
    }
    /***Clipboard*********************************/
    public Node copy(int start, int end) throws BadLocationException {
	if (start>=end) 
	    return null;
	Element el = root.getDeepElementAt(start, end);
	if (el!=null && el instanceof NodeSwingElement) {
	    org.w3c.dom.Document doc = DOMUtil.createDocument();
	    org.w3c.dom.Element xel = doc.createElement(((SwingBody)root).getTagName());
	    doc.appendChild(xel);
	    Node n = ((NodeSwingElement)el).copyXmlNode(doc, start, end);
	    if (n!=null) {
		xel.appendChild(n);
		return xel;
	    }
	}
	return null;
    }
    protected NodeSwingElement findPasteDestination(Element sel, int pos, Node n)
	throws BadLocationException {
	if (sel instanceof NodeSwingElement) {
	    NodeSwingElement nse = (NodeSwingElement)sel;
	    // paste in children?
	    if (!nse.isLeaf() && nse.getElementCount()>0) {
		int idx = nse.getIndexAt(pos, Position.Bias.Forward);
		if (Debug.DEBUG && idx==-1)
		    throw new InternalError("idx="+idx);
		Element ch = nse.getElementAtIndex(idx);
		NodeSwingElement ret = findPasteDestination(ch, pos, n);
		if (ret!=null) return ret;
		if (pos==ch.getEndOffset() && idx<(nse.getElementCount()-1)) {
		    idx++;
		    ch = nse.getElementAtIndex(idx);
		    ret = findPasteDestination(ch, pos, n);
		    if (ret!=null) return ret;
		}
	    }
	    if (n instanceof org.w3c.dom.Element) {
		// can paste into nse?
		if (nse.canInsertAt(pos)) {
		    Node xn = nse.getXmlNode();
		    if (n.getNodeType()==xn.getNodeType() &&
			n.getNodeName().equals(xn.getNodeName()))
			return nse;
		    if (xn instanceof org.w3c.dom.Element) {
			Hashtable ss = styles.getStyle(n.getNodeName());
			Hashtable ds = styles.getStyle(xn.getNodeName());
			if (ss!=null && ds!=null) {
			    Integer sci = (Integer)ss.get(styles.CLASS);
			    Integer dci = (Integer)ds.get(styles.CLASS);
			    final int sc = sci.intValue();
			    final int dc = dci.intValue();
			    if (((sc==styles.C_SECTION || sc==styles.C_UL ||
				  sc==styles.C_OL) &&
				 (dc==styles.C_SECTION || dc==styles.C_UL ||
				  dc==styles.C_OL)) ||
				(sc==styles.C_PARAGRAPH && dc==styles.C_PARAGRAPH) ||
				(sc==styles.C_TEXT && dc==styles.C_TEXT))
				return nse;
			}
		    }
		}
	    } else if (n instanceof org.w3c.dom.Text) {
		if (nse instanceof SwingText)
		    return nse;
	    }
	}
	return null;
    }
    public boolean paste(int pos, Node el) throws BadLocationException {
	beginUpdate();
	try {
	    DocumentEvent evt = new DocumentEvent(this);
	    NodeSwingElement nse = findPasteDestination(root, pos, el);
	    if (nse!=null && nse.pasteNode(pos, el, evt) && evt.isValid()) {
		fireDocumentChanged(evt); return true;
	    }
	    SwingElementFactory fac = getSwingElementFactory();
	    NodeSwingElement newChild = fac.create(null, el);
	    if (newChild!=null) {
		if (root.pasteElementAt(pos, newChild, evt)) {
		    fireDocumentChanged(evt); return true;
		} else if (!newChild.isLeaf()) {
		    boolean ok = false;
		    for (int z=newChild.getElementCount()-1; z>=0; z--) {
			Element ch = newChild.getElementAtIndex(z);
			if (ch instanceof NodeSwingElement &&
			    newChild.childConservationPossible(ch)) {
			    newChild.removeChild(ch);
			    if (paste(pos, ((NodeSwingElement)ch).getXmlNode()))
				ok = true;
			}
		    }
		    return ok;
		}
	    }
	    return false;
	} finally {
	    endUpdate();
	}
    }
    /***Find***************************************/
    protected Paragraph getParagraph(Element el, int pos) 
	throws BadLocationException {
	if (!el.isLeaf() && el.getElementCount()>0) {
	    int idx = el.getIndexAt(pos, Position.Bias.Forward);
	    if (idx>=0) {
		Paragraph p = getParagraph(el.getElementAtIndex(idx), pos);
		if (p!=null)
		    return p;
	    }
	}
	if (el instanceof Paragraph)
	    return (Paragraph)el;
	return null;
    }
    public Paragraph getParagraph(int pos) 
	throws BadLocationException {
	return getParagraph(root, pos);
    }
    public org.w3c.dom.Document getXmlDocument() { 
	return editorDocument.getNode().getOwnerDocument(); 
    }
    /***public**************************************/
    public SwingElementFactory getSwingElementFactory() {
	return elementFactory;
    }
    public EditorDocument getEditorDocument() {
	return editorDocument;
    }
    public DocumentType getDocumentType() { 
	return editorDocument.getType();
    }
    public EmbeddedManager getEmbeddedManager() {
	return editorDocument.getEmbeddedManager();
    }
}
